@@ -40,7 +40,7 @@
 #define SYSTEM_TRAY_CANCEL_MESSAGE  2
 #endif
 
-#define STATUSBAR_DEBUG 1
+#define STATUSBAR_DEBUG 0
 #if (STATUSBAR_DEBUG)
 #define sb_debug(o,a...) g_debug("sb: " o,##a)
 #else
@@ -306,6 +306,17 @@ hildon_desktop_panel_expandable_grab_table_focus (GtkWidget *table_window,
   return FALSE;  
 }
 
+static gboolean
+hildon_desktop_panel_expandable_hide_on_delete (GtkWidget *extension_window,
+						GdkEvent *event,
+						gpointer *panel)
+{
+  hildon_desktop_panel_expandable_hide_extension
+    (HILDON_DESKTOP_PANEL_EXPANDABLE (panel));
+
+  return TRUE;
+}
+
 static GObject *
 hildon_desktop_panel_expandable_constructor (GType gtype,
 			                     guint n_params, 
@@ -327,8 +338,14 @@ hildon_desktop_panel_expandable_constructor (GType gtype,
   gtk_window_set_type_hint (panel->priv->extension_window,
 		  	    GDK_WINDOW_TYPE_HINT_DIALOG);
 
+#ifdef MAEMO_CHANGES
+  gtk_window_set_is_temporary (panel->priv->extension_window, TRUE);
+#endif
+  
   gtk_window_set_decorated (panel->priv->extension_window, FALSE);
 
+  gtk_window_set_accept_focus (panel->priv->extension_window, FALSE);
+
   panel->priv->extension_table = GTK_TABLE (gtk_table_new (1,panel->priv->items_p_row,TRUE));
 
   g_object_set (panel->priv->extension_table,
@@ -352,6 +369,11 @@ hildon_desktop_panel_expandable_constructor (GType gtype,
 			  G_CALLBACK (hildon_desktop_panel_expandable_grab_table_focus),
 			  (gpointer)panel);
 
+  g_signal_connect (panel->priv->extension_window,
+		    "delete-event",
+		    G_CALLBACK (hildon_desktop_panel_expandable_hide_on_delete),
+		    (gpointer)panel);
+
   g_signal_connect (panel->priv->extension_window,
 		    "button-release-event",
 		    G_CALLBACK (hildon_desktop_panel_expandable_extension_winevent),
@@ -535,8 +557,10 @@ hildon_desktop_panel_expandable_add_button (HildonDesktopPanel *panel, GtkWidget
   gint item_width,item_height;
   GtkRequisition req;
 
+  g_return_if_fail (GTK_IS_WIDGET (button));
+
   if (ex_panel->priv->extension_opened)
-    hildon_desktop_panel_expandable_hide_extension (ex_panel);	  
+    hildon_desktop_panel_expandable_hide_extension (ex_panel);
 
   g_signal_emit_by_name (ex_panel, "queued-button", button);
 
@@ -551,7 +575,8 @@ hildon_desktop_panel_expandable_add_button (HildonDesktopPanel *panel, GtkWidget
     item_width = req.width;
   
   /* gtk_widget_set_size_request (button, item_width, item_height);*/
-  gtk_widget_set_size_request (GTK_BIN (button)->child, item_width, item_height);
+  if (GTK_IS_BIN (button) && GTK_IS_WIDGET (GTK_BIN (button)->child))
+    gtk_widget_set_size_request (GTK_BIN (button)->child, item_width, item_height);
   
   g_object_set (G_OBJECT (item), "position", ex_panel->priv->current_position++, NULL);
   
@@ -1083,7 +1108,7 @@ hildon_desktop_panel_embed_applet (HildonDesktopPanelExpandable *panel, Window w
 GtkWidget *
 hildon_desktop_panel_expandable_get_extension (HildonDesktopPanelExpandable *panel)
 {
-  g_assert (HILDON_DESKTOP_IS_PANEL_EXPANDABLE (panel));
+  g_return_val_if_fail (HILDON_DESKTOP_IS_PANEL_EXPANDABLE (panel), NULL);
 
   return GTK_WIDGET (panel->priv->extension_table);
 }
