@@ -4,6 +4,7 @@
  * Copyright (C) 2007 Nokia Corporation.
  *
  * Author:  Moises Martinez <moises.martinez@nokia.com>
+ *          Lucas Rocha <lucas.rocha@nokia.com>
  * Contact: Karoliina Salminen <karoliina.t.salminen@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
@@ -55,6 +56,8 @@ enum
 
 static gint signals[N_SIGNALS];
 
+#define MENU_SCROLL_TIMEOUT 40
+
 struct _HildonDesktopPopupMenuPrivate
 {
   GtkWidget    *parent;
@@ -76,6 +79,11 @@ struct _HildonDesktopPopupMenuPrivate
   gboolean      resize_parent;
 
   guint         toggle_size;
+
+  gboolean      rtl;
+
+  gint          scroll_id;
+  gint          scroll_dir;
 };
 
 static GObject *hildon_desktop_popup_menu_constructor (GType gtype,
@@ -97,9 +105,14 @@ static void hildon_desktop_popup_menu_set_property (GObject *object,
 static void hildon_desktop_popup_menu_size_request (GtkWidget *widget, GtkRequisition *req);
 static void hildon_desktop_popup_menu_size_allocate (GtkWidget *widget, GtkAllocation *allocation);
 static gboolean hildon_desktop_popup_menu_motion_notify (GtkWidget *widget, GdkEventMotion *event);
+static gboolean hildon_desktop_popup_menu_press_event (GtkWidget *widget, GdkEventButton *event);
 static gboolean hildon_desktop_popup_menu_release_event (GtkWidget *widget, GdkEventButton *event);
 static gboolean hildon_desktop_popup_menu_key_press_event (GtkWidget *widget, GdkEventKey *event);
+static void hildon_desktop_popup_menu_select_next_prev_item (HildonDesktopPopupMenu *menu, gboolean next);
 static void hildon_desktop_popup_menu_scroll_cb (GtkWidget *widget, HildonDesktopPopupMenu *menu);
+static void hildon_desktop_popup_menu_scroll_start (GtkWidget *widget, HildonDesktopPopupMenu *menu);
+static void hildon_desktop_popup_menu_scroll_stop (GtkWidget *widget, HildonDesktopPopupMenu *menu);
+static void hildon_desktop_popup_menu_adjustment_changed (GtkAdjustment *adj, HildonDesktopPopupMenu *menu);
 
 static void 
 hildon_desktop_popup_menu_init (HildonDesktopPopupMenu *menu)
@@ -133,6 +146,7 @@ hildon_desktop_popup_menu_class_init (HildonDesktopPopupMenuClass *menu_class)
   object_class->set_property = hildon_desktop_popup_menu_set_property;
 
   widget_class->motion_notify_event  = hildon_desktop_popup_menu_motion_notify;
+  widget_class->button_press_event = hildon_desktop_popup_menu_press_event;
   widget_class->button_release_event = hildon_desktop_popup_menu_release_event;
   widget_class->key_press_event      = hildon_desktop_popup_menu_key_press_event;
   
@@ -215,14 +229,16 @@ hildon_desktop_popup_menu_constructor (GType gtype,
                                        guint n_params,
                                        GObjectConstructParam *params)
 {
-  GObject *object;
   HildonDesktopPopupMenu *menu;
+  GtkAdjustment *adj;
+  GObject *object;
 
   object = G_OBJECT_CLASS (hildon_desktop_popup_menu_parent_class)->constructor (gtype,
                                                                                  n_params,
      										 params);
   menu = HILDON_DESKTOP_POPUP_MENU (object);
 
+  menu->priv->rtl = (gtk_widget_get_direction (GTK_WIDGET (menu)) == GTK_TEXT_DIR_RTL);
   menu->priv->viewport = gtk_viewport_new (NULL, NULL);
 
   gtk_box_pack_start (GTK_BOX (menu),
@@ -231,10 +247,24 @@ hildon_desktop_popup_menu_constructor (GType gtype,
 
   gtk_widget_show (menu->priv->viewport);
 
+  adj = gtk_viewport_get_vadjustment (GTK_VIEWPORT (menu->priv->viewport));
+  
+  g_signal_connect (adj,
+                    "value-changed",
+                    G_CALLBACK (hildon_desktop_popup_menu_adjustment_changed), 
+                    menu);
+
+  g_signal_connect (adj,
+                    "changed",
+                    G_CALLBACK (hildon_desktop_popup_menu_adjustment_changed), 
+                    menu);
+
   gtk_widget_push_composite_child ();
 
   menu->priv->box_items = gtk_vbox_new (FALSE, 0); /* FIXME: add spacing decoration */
   
+  gtk_widget_set_size_request (menu->priv->box_items, 1, -1);
+
   gtk_container_add (GTK_CONTAINER (menu->priv->viewport), menu->priv->box_items);
 
   gtk_widget_show (menu->priv->box_items);
@@ -244,22 +274,50 @@ hildon_desktop_popup_menu_constructor (GType gtype,
   menu->priv->scroll_down = gtk_button_new ();
   menu->priv->scroll_up   = gtk_button_new ();
 
+  g_object_set (G_OBJECT (menu->priv->scroll_up),
+		"can-focus", FALSE,
+		NULL);
+  
+  g_object_set (G_OBJECT (menu->priv->scroll_down),
+		"can-focus", FALSE,
+		NULL);
+  
   g_signal_connect (menu->priv->scroll_down,
 		    "clicked",
 		    G_CALLBACK (hildon_desktop_popup_menu_scroll_cb),
 		    (gpointer)menu);
 
+  g_signal_connect (menu->priv->scroll_down,
+		    "pressed",
+		    G_CALLBACK (hildon_desktop_popup_menu_scroll_start),
+		    (gpointer)menu);
+
+  g_signal_connect (menu->priv->scroll_down,
+		    "released",
+		    G_CALLBACK (hildon_desktop_popup_menu_scroll_stop),
+		    (gpointer)menu);
+
   g_signal_connect (menu->priv->scroll_up,
                     "clicked",
                     G_CALLBACK (hildon_desktop_popup_menu_scroll_cb),
                     (gpointer)menu);		    
 
+  g_signal_connect (menu->priv->scroll_up,
+		    "pressed",
+		    G_CALLBACK (hildon_desktop_popup_menu_scroll_start),
+		    (gpointer)menu);
+
+  g_signal_connect (menu->priv->scroll_up,
+		    "released",
+		    G_CALLBACK (hildon_desktop_popup_menu_scroll_stop),
+		    (gpointer)menu);
+
   gtk_box_pack_start (GTK_BOX (menu->priv->box_buttons),
-		      menu->priv->scroll_down,
+		      menu->priv->scroll_up,
 		      TRUE, TRUE, 0);
   
   gtk_box_pack_start (GTK_BOX (menu->priv->box_buttons),
-		      menu->priv->scroll_up,
+		      menu->priv->scroll_down,
 		      TRUE, TRUE, 0);
   
   gtk_widget_show (menu->priv->scroll_down);
@@ -287,7 +345,7 @@ static void
 hildon_desktop_popup_menu_finalize (GObject *object)
 {
   g_object_unref (G_OBJECT (HILDON_DESKTOP_POPUP_MENU (object)->priv->box_buttons));
-	
+
   G_OBJECT_CLASS (hildon_desktop_popup_menu_parent_class)->finalize (object);	
 }	
 
@@ -319,16 +377,19 @@ hildon_desktop_popup_menu_size_allocate (GtkWidget *widget, GtkAllocation *alloc
 
       child_allocation.width = allocation->width;
       child_allocation.height = child_requisition.height;
-      child_allocation.x = 0;
+
+      if (priv->rtl)
+        child_allocation.x = child_allocation.width;
+      else
+        child_allocation.x = 0;
+
       child_allocation.y = height;
-	    
+
       gtk_menu_item_toggle_size_allocate (GTK_MENU_ITEM (child),
                                           priv->toggle_size);
 
       gtk_widget_size_allocate (child, &child_allocation);
 
-      gtk_widget_queue_draw (GTK_WIDGET (child));
-
       height += child_requisition.height;
     }
   }
@@ -457,9 +518,6 @@ hildon_desktop_popup_menu_motion_notify (GtkWidget      *widget,
   menu_items = 
     gtk_container_get_children (GTK_CONTAINER (menu->priv->box_items));
 
-  for (l = menu_items; l != NULL; l = g_list_next (l))
-    gtk_item_deselect (GTK_ITEM (l->data));	  
-	
   for (l = menu_items; l != NULL; l = g_list_next (l))
   {
     gtk_widget_get_pointer (GTK_WIDGET (l->data), &x, &y);
@@ -468,9 +526,15 @@ hildon_desktop_popup_menu_motion_notify (GtkWidget      *widget,
     h = GTK_WIDGET (l->data)->allocation.height;
 
     if ((x >= 0) && (x <= w) && (y >= 0) && (y <= h))
-    {	    
-      gtk_item_select (GTK_ITEM (l->data));
-      menu->priv->selected_item = GTK_MENU_ITEM (l->data);
+    {
+      if (menu->priv->selected_item != GTK_MENU_ITEM (l->data))
+      {
+        gtk_item_deselect (GTK_ITEM (menu->priv->selected_item));
+        gtk_item_select (GTK_ITEM (l->data));
+        menu->priv->selected_item = GTK_MENU_ITEM (l->data);
+      }
+
+      break;
     }
   }
 
@@ -480,44 +544,102 @@ hildon_desktop_popup_menu_motion_notify (GtkWidget      *widget,
 }
 
 static void 
-hildon_desktop_popup_menu_scroll_cb (GtkWidget *widget, HildonDesktopPopupMenu *menu)
+hildon_desktop_popup_menu_update_scroll_buttons (HildonDesktopPopupMenu *menu)
 {
-  GtkRequisition req;
   GtkAdjustment *adj;
-  gdouble position;
+
+  adj = gtk_viewport_get_vadjustment (GTK_VIEWPORT (menu->priv->viewport));
+
+  gtk_widget_set_sensitive (menu->priv->scroll_up, 
+		            (adj->value > 0));
+
+  gtk_widget_set_sensitive (menu->priv->scroll_down, 
+		            (adj->value < adj->upper - adj->page_size));
+}
+
+static void 
+hildon_desktop_popup_menu_adjustment_changed (GtkAdjustment *adj, 
+					      HildonDesktopPopupMenu *menu)
+{
+  hildon_desktop_popup_menu_update_scroll_buttons (menu);
+}
+
+static gboolean 
+hildon_desktop_popup_menu_scroll_step (gpointer data)
+{
+  HildonDesktopPopupMenu *menu = HILDON_DESKTOP_POPUP_MENU (data);
+  GtkAdjustment *adj;
   gint delta = menu->priv->item_height + 1;
  
   adj = gtk_viewport_get_vadjustment (GTK_VIEWPORT (menu->priv->viewport));
 
-  if (widget == menu->priv->scroll_up)
-    delta *= -1;
+  delta *= menu->priv->scroll_dir;
   
-  position = gtk_adjustment_get_value (adj);
-
-  if (menu->priv->parent)
-  {		  
-    gtk_widget_size_request (menu->priv->parent, &req);
- 
-    if ((gint) (position + (gdouble) delta) <= adj->upper - adj->page_size)
-      gtk_adjustment_set_value (adj, position + (gdouble) delta); 
-    else
-      gtk_adjustment_set_value (adj, adj->upper - adj->page_size);
-
-    /* NOTE: Don't remove this 
-    g_debug ("min: %lf max: %lf current: %lf", adj->lower,adj->upper, adj->value);*/
+  if ((gint) (adj->value + (gdouble) delta) >= 0 &&
+      (gint) (adj->value + (gdouble) delta) <= adj->upper - adj->page_size)
+  {
+    adj->value += (gdouble) delta; 
+    gtk_adjustment_value_changed (adj);
   }
+  else
+  {
+    if (delta > 0)
+      adj->value = adj->upper - adj->page_size;
+    else
+      adj->value = 0;
+
+    gtk_adjustment_value_changed (adj);
+    return FALSE;
+  }
+
+  return TRUE;
+}
+
+static void 
+hildon_desktop_popup_menu_scroll_start (GtkWidget *widget, HildonDesktopPopupMenu *menu)
+{
+  if (widget == menu->priv->scroll_up)
+    menu->priv->scroll_dir = -1;
+  else
+    menu->priv->scroll_dir = 1;
+
+  menu->priv->scroll_id = 
+    g_timeout_add (MENU_SCROLL_TIMEOUT, hildon_desktop_popup_menu_scroll_step, menu);
+}
+
+static void 
+hildon_desktop_popup_menu_scroll_stop (GtkWidget *widget, HildonDesktopPopupMenu *menu)
+{
+  g_source_remove (menu->priv->scroll_id);
+  menu->priv->scroll_id = 0;
+}
+
+static void 
+hildon_desktop_popup_menu_scroll_cb (GtkWidget *widget, HildonDesktopPopupMenu *menu)
+{
+  if (widget == menu->priv->scroll_up)
+    menu->priv->scroll_dir = -1;
+  else
+    menu->priv->scroll_dir = 1;
+
+  hildon_desktop_popup_menu_scroll_step (menu);
 }
 
 static gboolean 
 hildon_desktop_popup_menu_release_event (GtkWidget      *widget,
                                          GdkEventButton *event)
+{
+  return TRUE;
+} 
+	
+static gboolean 
+hildon_desktop_popup_menu_press_event (GtkWidget      *widget,
+                                       GdkEventButton *event)
 {
   HildonDesktopPopupMenu *menu = HILDON_DESKTOP_POPUP_MENU (widget);
   GList *menu_items = NULL, *l;
   gint w,h,x,y;
 
-  g_debug ("release event for popup menu");
-  
   menu_items =
     gtk_container_get_children (GTK_CONTAINER (menu->priv->box_items));
 
@@ -532,11 +654,18 @@ hildon_desktop_popup_menu_release_event (GtkWidget      *widget,
     {
       if (GTK_IS_MENU_ITEM (l->data))
       {
-        gtk_menu_item_activate (GTK_MENU_ITEM (l->data));
+        if (menu->priv->selected_item != GTK_MENU_ITEM (l->data))
+        {
+          gtk_item_deselect (GTK_ITEM (menu->priv->selected_item));
+          gtk_item_select (GTK_ITEM (l->data));
+          menu->priv->selected_item = GTK_MENU_ITEM (l->data);
+        }
+        
+	gtk_menu_item_activate (GTK_MENU_ITEM (l->data));
         break;
       }
     }
-   }
+  }
 
   g_list_free (menu_items);
 
@@ -544,30 +673,41 @@ hildon_desktop_popup_menu_release_event (GtkWidget      *widget,
 }
 
 static void 
-hildon_desktop_menu_check_scroll_item (HildonDesktopPopupMenu *menu,
-				       GtkScrollType type)
+hildon_desktop_menu_check_scroll_item (HildonDesktopPopupMenu *menu)
 {
-  GtkAdjustment *adj;	
-  gint visible_y;
-  GtkWidget *view = GTK_WIDGET (menu->priv->selected_item);
-  gdouble upper_hack; 
+  GtkWidget *view = menu->priv->box_items;
+  GtkWidget *menu_item = GTK_WIDGET (menu->priv->selected_item);
+  GtkWidget *viewport = menu->priv->viewport;
+  GtkAdjustment *adj;
   GtkRequisition req;
+  gint visible_y;
 
+  if (menu_item == NULL) 
+    return;
+  
   gtk_widget_size_request (menu->priv->parent, &req);
 
   adj = gtk_viewport_get_vadjustment (GTK_VIEWPORT (menu->priv->viewport));
   
-  if (menu->priv->parent) 
-    upper_hack = adj->upper - (req.height - menu->priv->item_height + 1);
-  else
-    upper_hack = adj->upper;	  
+  g_return_if_fail ((adj->upper - adj->lower) && view->allocation.height);
 
-  visible_y = view->allocation.y + menu->priv->item_height;
- 
-  if (visible_y < (req.height - menu->priv->item_height)) 
-    hildon_desktop_popup_menu_scroll_cb (menu->priv->scroll_up, menu);
-  else
-    hildon_desktop_popup_menu_scroll_cb (menu->priv->scroll_down, menu);
+  visible_y = view->allocation.y +
+     (gint)(view->allocation.height * adj->value / (adj->upper - adj->lower));
+
+  if (menu_item->allocation.y < visible_y)
+  {
+    adj->value = menu_item->allocation.y * (adj->upper - adj->lower)
+                                        / view->allocation.height;
+    gtk_adjustment_value_changed (adj);
+  }
+  else if (menu_item->allocation.y + menu_item->allocation.height > 
+           visible_y + viewport->allocation.height)
+  {
+    adj->value = (menu_item->allocation.y + menu_item->allocation.height
+           - viewport->allocation.height) * (adj->upper - adj->lower)
+           / view->allocation.height;
+    gtk_adjustment_value_changed (adj);
+  }
 }
 
 static gboolean 
@@ -575,67 +715,20 @@ hildon_desktop_popup_menu_key_press_event (GtkWidget   *widget,
 					   GdkEventKey *event)
 {
   HildonDesktopPopupMenu *menu = HILDON_DESKTOP_POPUP_MENU (widget);
-  GList *menu_items = NULL, *l;
-
-  menu_items =
-    gtk_container_get_children (GTK_CONTAINER (menu->priv->box_items)); 
-
-  for (l = menu_items; l != NULL; l = g_list_next (l))
-  {
-    if (l->data == menu->priv->selected_item)
-      break;
-  }
-
-  if (l == NULL)
-    return FALSE;   
 
   if (event->keyval == GDK_Up ||
       event->keyval == GDK_KP_Up)
   {
-    GList *item = l->prev;
-	  
-    while (item)
-    {
-      if (GTK_IS_MENU_ITEM (item->data) && !GTK_IS_SEPARATOR_MENU_ITEM (item->data))
-      {
-        gtk_item_deselect (GTK_ITEM (l->data));
-        gtk_item_select   (GTK_ITEM (item->data));
-	menu->priv->selected_item = GTK_MENU_ITEM (item->data);
-
-	if (menu->priv->controls_on)
-  	  hildon_desktop_menu_check_scroll_item (menu, GTK_SCROLL_STEP_UP);
-	
-	break;
-      }
-
-      item = g_list_previous (item);
-    } 
+    hildon_desktop_popup_menu_select_next_prev_item (menu, FALSE);
     return TRUE;
   }
   else
   if (event->keyval == GDK_Down ||
       event->keyval == GDK_KP_Down)
   {
-    GList *item = l->next;
-    
-    while  (item)    
-    {	   
-      if (GTK_IS_MENU_ITEM (item->data) && !GTK_IS_SEPARATOR_MENU_ITEM (item->data))
-      { 
-        gtk_item_deselect (GTK_ITEM (l->data));
-        gtk_item_select   (GTK_ITEM (item->data));
-	menu->priv->selected_item = GTK_MENU_ITEM (item->data);
-
-	if (menu->priv->controls_on)
-	  hildon_desktop_menu_check_scroll_item (menu, GTK_SCROLL_STEP_DOWN);
-	
-	break;
-      }
-
-      item = g_list_next (item);
-    }
+    hildon_desktop_popup_menu_select_next_prev_item (menu, TRUE);
     return TRUE;
-  }	  
+  }
   else
   if (event->keyval == GDK_Return   ||
       event->keyval == GDK_KP_Enter ||
@@ -656,10 +749,13 @@ hildon_desktop_popup_menu_show_controls (HildonDesktopPopupMenu *menu)
     gtk_box_pack_start (GTK_BOX (menu),
 		        menu->priv->box_buttons,
 		        FALSE, FALSE, 0);
+
     gtk_widget_show (menu->priv->box_buttons);
-    
+
     menu->priv->controls_on = TRUE;
 
+    hildon_desktop_popup_menu_update_scroll_buttons (menu);
+
     g_signal_emit_by_name (menu, "show-controls", TRUE);
   }
 }
@@ -733,10 +829,8 @@ hildon_desktop_popup_menu_parent_size (HildonDesktopPopupMenu *menu)
 		  "border-width", &border_width,
 		  NULL); 
     
-    gtk_widget_size_request (menu->priv->parent, &req);
-
     gtk_widget_set_size_request (menu->priv->parent,
-		    		 req.width,
+		    		 parent_req.width,
 				 show_scroll_controls ? 
 				 max_height : 
 				 d_height + (2 * border_width));
@@ -751,7 +845,7 @@ hildon_desktop_popup_menu_parent_size (HildonDesktopPopupMenu *menu)
     
     if (GTK_WIDGET_REALIZED (menu->priv->parent))
       gdk_window_resize (menu->priv->parent->window,
-		         req.width,
+		         parent_req.width,
 			 show_scroll_controls ? 
 			 max_height : 
 	                 d_height + (2 * border_width));
@@ -772,7 +866,7 @@ hildon_desktop_popup_menu_select_next_prev_item (HildonDesktopPopupMenu *menu, g
   previous_selected_item = menu->priv->selected_item;
 
   children = gtk_container_get_children (GTK_CONTAINER (menu->priv->box_items));
-
+  
   if (children)
   {
     l = item = g_list_find (children, menu->priv->selected_item);
@@ -786,7 +880,7 @@ hildon_desktop_popup_menu_select_next_prev_item (HildonDesktopPopupMenu *menu, g
           hildon_desktop_popup_menu_select_item (menu, GTK_MENU_ITEM (l->data));
 	  break;
         }		
-      }	      
+      }	
 
       if (next)
         l = l->next;
@@ -794,9 +888,14 @@ hildon_desktop_popup_menu_select_next_prev_item (HildonDesktopPopupMenu *menu, g
 	l = l->prev;      
     }	    
 
-    if (previous_selected_item == menu->priv->selected_item) /* TODO: This only cover one case. */
-      hildon_desktop_popup_menu_select_first_item (menu);    /* It doesn't take into account the direction */
-	    
+    if (previous_selected_item == menu->priv->selected_item)
+    {
+      if (next) 
+        hildon_desktop_popup_menu_select_first_item (menu); 
+      else
+        hildon_desktop_popup_menu_select_last_item (menu); 
+    }
+      
     g_list_free (children);
   }	  
 }
@@ -816,11 +915,16 @@ hildon_desktop_popup_menu_add_item (HildonDesktopPopupMenu *menu, GtkMenuItem *i
       item_width = req.width;
   }
   
+  gtk_box_pack_end (GTK_BOX (menu->priv->box_items),
+		    GTK_WIDGET (item),
+		    FALSE, FALSE, 0);
+
   if (GTK_IS_SEPARATOR_MENU_ITEM (item))
   {
     GtkRequisition req_sep;
 
     gtk_widget_size_request (GTK_WIDGET (item), &req_sep);
+
     gtk_widget_set_size_request (GTK_WIDGET (item), item_width, req_sep.height);
   }
   else
@@ -828,15 +932,11 @@ hildon_desktop_popup_menu_add_item (HildonDesktopPopupMenu *menu, GtkMenuItem *i
     gtk_widget_set_size_request (GTK_WIDGET (item), item_width, menu->priv->item_height);
   }
   
-  gtk_box_pack_end (GTK_BOX (menu->priv->box_items),
-		    GTK_WIDGET (item),
-		    FALSE, FALSE, 0);
-
   gtk_widget_show (GTK_WIDGET (item));
 
   gtk_widget_set_size_request (menu->priv->box_buttons, 
 		  	       item_width, 
-			       menu->priv->item_height);
+			       menu->priv->item_height + 2);
 
   menu->priv->n_items++;
 
@@ -869,24 +969,15 @@ hildon_desktop_popup_menu_get_children (HildonDesktopPopupMenu *menu)
 void 
 hildon_desktop_popup_menu_select_item (HildonDesktopPopupMenu *menu, GtkMenuItem *item)
 {
-  GList *children = NULL, *l;
-
   g_return_if_fail (HILDON_DESKTOP_IS_POPUP_MENU (menu));
+  g_return_if_fail (GTK_IS_MENU_ITEM (item));
 
-  children = gtk_container_get_children (GTK_CONTAINER (menu->priv->box_items));
+  if (GTK_IS_ITEM (menu->priv->selected_item))
+    gtk_item_deselect (GTK_ITEM (menu->priv->selected_item));	    
 
-  for (l = children; l != NULL; l = g_list_next (l))
-  {
-    if (l->data && l->data == item)
-    {
-      gtk_item_select (GTK_ITEM (item));
-      menu->priv->selected_item = item;
-    }
-    else
-      gtk_item_deselect (GTK_ITEM (l->data));	    
-  }
-
-  g_list_free (children);
+  gtk_item_select (GTK_ITEM (item));
+  menu->priv->selected_item = item;
+  hildon_desktop_menu_check_scroll_item (menu);
 } 
 
 void 
@@ -914,14 +1005,41 @@ hildon_desktop_popup_menu_select_first_item (HildonDesktopPopupMenu *menu)
   {	  
     if (!GTK_IS_SEPARATOR_MENU_ITEM (l->data))
     {
-      if (menu->priv->selected_item != NULL)
+      if (GTK_IS_ITEM (menu->priv->selected_item))
         gtk_item_deselect (GTK_ITEM (menu->priv->selected_item));
 
       gtk_item_select (GTK_ITEM (l->data));
       menu->priv->selected_item = GTK_MENU_ITEM (l->data);
+      hildon_desktop_menu_check_scroll_item (menu);
+
       break;
     }	    
-  }	  
+  }
+}	
+
+void
+hildon_desktop_popup_menu_select_last_item (HildonDesktopPopupMenu *menu)
+{
+  GList *children = NULL, *l;
+
+  g_return_if_fail (HILDON_DESKTOP_IS_POPUP_MENU (menu));
+
+  children = gtk_container_get_children (GTK_CONTAINER (menu->priv->box_items));
+
+  for (l = g_list_last (children); l != NULL; l = g_list_previous (l))
+  {
+    if (!GTK_IS_SEPARATOR_MENU_ITEM (l->data))
+    {
+      if (GTK_IS_ITEM (menu->priv->selected_item))
+        gtk_item_deselect (GTK_ITEM (menu->priv->selected_item));
+
+      gtk_item_select (GTK_ITEM (l->data));
+      menu->priv->selected_item = GTK_MENU_ITEM (l->data);
+      hildon_desktop_menu_check_scroll_item (menu);
+
+      break;
+    }
+  }
 }	
 
 void
@@ -937,7 +1055,7 @@ hildon_desktop_popup_menu_activate_item (HildonDesktopPopupMenu *menu, GtkMenuIt
   {
     if (l->data == item)
     {
-      gtk_menu_item_activate (GTK_MENU_ITEM (item));
+      gtk_menu_item_activate (GTK_MENU_ITEM (item));      
       break;
     }
   }
@@ -966,38 +1084,13 @@ hildon_desktop_popup_menu_select_prev_item (HildonDesktopPopupMenu *menu)
 void   
 hildon_desktop_popup_menu_scroll_to_selected (HildonDesktopPopupMenu *menu)
 {
-  GList *children = NULL, *l;
-  gint position = 0, screen_height = gdk_screen_height (); 	 
- 
-  if (!HILDON_DESKTOP_IS_POPUP_MENU (menu))
-    return;
+  g_return_if_fail (HILDON_DESKTOP_IS_POPUP_MENU (menu));
 
   if (!menu->priv->controls_on)
     return;	  
  
   if (menu->priv->selected_item)
-  {    	  
-    children = gtk_container_get_children (GTK_CONTAINER (menu->priv->box_items));
-	  
-    for (l=children; l != NULL; l = g_list_next (l))
-    {
-      position += menu->priv->item_height;
-      
-      if (l->data == menu->priv->selected_item)
-        break;
-    }
-
-    if (position > screen_height)
-    {
-      do
-      {	      
-        hildon_desktop_popup_menu_scroll_cb (menu->priv->scroll_up, menu);
-	position -= menu->priv->item_height;
-      }	
-      while (position > screen_height);
-    }	    
-
-  }
+    hildon_desktop_menu_check_scroll_item (menu);
 }
 
 const GtkWidget *
