@@ -42,10 +42,22 @@
 #include <string.h>
 #include <stdlib.h> /* malloc */
 
+/* .desktop keys */
+#define HH_APPLET_GROUP                 "Desktop Entry"
+#define HH_APPLET_KEY_NAME              "Name"
+#define HH_APPLET_KEY_LIBRARY           "X-home-applet"
+#define HH_APPLET_KEY_X                 "X"
+#define HH_APPLET_KEY_Y                 "Y"
+#define HH_APPLET_KEY_WIDTH             "X-home-applet-width"
+#define HH_APPLET_KEY_HEIGHT            "X-home-applet-height"
+#define HH_APPLET_KEY_STACK_INDEX       "X-home-applet-stack-index"
+#define HH_APPLET_KEY_MINWIDTH          "X-home-applet-minwidth"
+#define HH_APPLET_KEY_MINHEIGHT         "X-home-applet-minheight"
+#define HH_APPLET_KEY_RESIZABLE         "X-home-applet-resizable"
+
 enum
 {
-  PROP_LAYOUT_MODE = 1,
-  PROP_SNAP_TO_GRID,
+  PROP_SNAP_TO_GRID = 1,
   PROP_APPLET_PADDING,
   PROP_APPLET_DEFAULT_ALPHA,
   PROP_CHILD_X,
@@ -63,7 +75,7 @@ typedef struct
   gint          stack_index;
   gulong        realize_handler;
 #ifdef HAVE_X_COMPOSITE
-  GtkAllocation old_allocation;
+  gint          mask_width, mask_height;
   Damage        damage;
   Picture       picture;
   Picture       alpha_mask;
@@ -84,7 +96,6 @@ typedef struct
 
 typedef struct HildonHomeAreaPriv_
 {
-  gboolean      layout_mode;
   gboolean      layout_changed;
 
   gboolean      snap_to_grid;
@@ -120,12 +131,6 @@ hildon_home_area_class_init (HildonHomeAreaClass *klass);
 static void
 hildon_home_area_init (HildonHomeArea *area);
 
-static void
-hildon_home_area_layout_mode_start (HildonHomeArea *area);
-
-static void
-hildon_home_area_layout_mode_end (HildonHomeArea *area);
-
 static void
 hildon_home_area_layout_changed (HildonHomeArea *area);
 
@@ -326,8 +331,6 @@ hildon_home_area_class_init (HildonHomeAreaClass *klass)
 
   g_type_class_add_private (klass, sizeof (HildonHomeAreaPriv));
 
-  klass->layout_mode_start = hildon_home_area_layout_mode_start;
-  klass->layout_mode_end   = hildon_home_area_layout_mode_end;
   klass->layout_changed    = hildon_home_area_layout_changed;
   klass->applet_added      = hildon_home_area_applet_added;
 
@@ -353,46 +356,6 @@ hildon_home_area_class_init (HildonHomeAreaClass *klass)
   object_class->get_property = hildon_home_area_get_property;
   object_class->finalize = hildon_home_area_finalize;
 
-  g_signal_new ("layout-mode-start",
-                G_OBJECT_CLASS_TYPE (object_class),
-                G_SIGNAL_RUN_LAST,
-                G_STRUCT_OFFSET (HildonHomeAreaClass, layout_mode_start),
-                NULL,
-                NULL,
-                g_cclosure_marshal_VOID__VOID,
-                G_TYPE_NONE,
-                0);
-
-  g_signal_new ("layout-mode-started",
-                G_OBJECT_CLASS_TYPE (object_class),
-                G_SIGNAL_RUN_FIRST,
-                G_STRUCT_OFFSET (HildonHomeAreaClass, layout_mode_started),
-                NULL,
-                NULL,
-                g_cclosure_marshal_VOID__VOID,
-                G_TYPE_NONE,
-                0);
-
-  g_signal_new ("layout-mode-end",
-                G_OBJECT_CLASS_TYPE (object_class),
-                G_SIGNAL_RUN_FIRST,
-                G_STRUCT_OFFSET (HildonHomeAreaClass, layout_mode_end),
-                NULL,
-                NULL,
-                g_cclosure_marshal_VOID__VOID,
-                G_TYPE_NONE,
-                0);
-
-  g_signal_new ("layout-mode-ended",
-                G_OBJECT_CLASS_TYPE (object_class),
-                G_SIGNAL_RUN_LAST,
-                G_STRUCT_OFFSET (HildonHomeAreaClass, layout_mode_ended),
-                NULL,
-                NULL,
-                g_cclosure_marshal_VOID__VOID,
-                G_TYPE_NONE,
-                0);
-
   g_signal_new ("applet-added",
                 G_OBJECT_CLASS_TYPE (object_class),
                 G_SIGNAL_RUN_FIRST,
@@ -447,18 +410,6 @@ hildon_home_area_class_init (HildonHomeAreaClass *klass)
                 G_TYPE_NONE,
                 0);
 
-
-  pspec =  g_param_spec_boolean ("layout-mode",
-                                 "Layout mode",
-                                 "Whether the home area is in layout "
-                                 "mode",
-                                 FALSE,
-                                 G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
-
-  g_object_class_install_property (object_class,
-                                   PROP_LAYOUT_MODE,
-                                   pspec);
-
   pspec =  g_param_spec_boolean ("snap-to-grid",
                                  "Snap to grid",
                                  "Whether applets should snap to grid",
@@ -563,11 +514,6 @@ hildon_home_area_set_property (GObject      *object,
 
   switch (property_id)
     {
-      case PROP_LAYOUT_MODE:
-          hildon_home_area_set_layout_mode (HILDON_HOME_AREA (object),
-                                            g_value_get_boolean (value));
-           break;
-
       case PROP_SNAP_TO_GRID:
            priv->snap_to_grid = g_value_get_boolean (value);
            break;
@@ -597,10 +543,6 @@ hildon_home_area_get_property (GObject      *object,
 
   switch (property_id)
     {
-      case PROP_LAYOUT_MODE:
-          g_value_set_boolean (value, priv->layout_mode);
-          break;
-
       case PROP_SNAP_TO_GRID:
            g_value_set_boolean (value, priv->snap_to_grid);
            break;
@@ -633,7 +575,7 @@ hildon_home_area_get_child_data (HildonHomeArea *area,
   GList                *list_item;
 
   list_item = g_list_find_custom (priv->children_data,
-                                  child, 
+                                  child,
                                   (GCompareFunc)find_by_widget);
 
 
@@ -702,6 +644,8 @@ hildon_home_area_get_child_property (GtkContainer    *container,
   HildonHomeArea       *area = HILDON_HOME_AREA (container);
   ChildData            *child_data;
 
+  g_return_if_fail (child->parent == GTK_WIDGET (container));
+
   child_data = hildon_home_area_get_child_data (area, child);
 
   g_return_if_fail (child_data);
@@ -852,34 +796,6 @@ hildon_home_area_unrealize (GtkWidget *widget)
 }
 #endif
 
-static void
-hildon_home_area_layout_mode_start (HildonHomeArea *area)
-{
-  HildonHomeAreaPriv      *priv;
-  priv = HILDON_HOME_AREA_GET_PRIVATE (area);
-
-  gtk_container_foreach (GTK_CONTAINER (area),
-                         (GtkCallback)hildon_desktop_home_item_set_layout_mode,
-                         (gpointer)TRUE);
-
-  g_signal_emit_by_name (area, "layout-mode-started");
-}
-
-static void
-hildon_home_area_layout_mode_end (HildonHomeArea *area)
-{
-  HildonHomeAreaPriv      *priv;
-  priv = HILDON_HOME_AREA_GET_PRIVATE (area);
-
-  gtk_container_foreach (GTK_CONTAINER (area),
-                        (GtkCallback)hildon_desktop_home_item_set_layout_mode,
-                         (gpointer)FALSE);
-  g_signal_emit_by_name (area, "layout-mode-ended");
-
-  priv->layout_changed = FALSE;
-
-}
-
 static void
 hildon_home_area_layout_changed (HildonHomeArea *area)
 {
@@ -897,13 +813,6 @@ hildon_home_area_applet_added (HildonHomeArea *area, GtkWidget *applet)
   g_return_if_fail (area);
 
   priv = HILDON_HOME_AREA_GET_PRIVATE (area);
-  if (priv->layout_mode)
-    {
-      if (HILDON_DESKTOP_IS_HOME_ITEM (applet))
-        hildon_desktop_home_item_set_layout_mode (HILDON_DESKTOP_HOME_ITEM (applet),
-                                                  TRUE);
-      g_signal_emit_by_name (area, "layout-changed");
-    }
 }
 
 static void
@@ -951,10 +860,6 @@ hildon_home_area_add (GtkContainer *area, GtkWidget *applet)
           hildon_home_area_put (HILDON_HOME_AREA (area), applet, 0, 0, G_MAXINT);
         }
 
-
-      if (priv->layout_mode)
-        hildon_desktop_home_item_set_layout_mode (HILDON_DESKTOP_HOME_ITEM (applet),
-                                            TRUE);
     }
 }
 
@@ -1053,12 +958,10 @@ hildon_home_area_child_size_allocate (GtkWidget        *child,
                            NULL);
 
   if (state == 0 && /* not moving or resizing */
-      (data->old_allocation.width  != allocation->width ||
-       data->old_allocation.height != allocation->height))
+      (data->mask_width  != allocation->width ||
+       data->mask_height != allocation->height))
     hildon_home_area_child_build_alpha_mask (area, child);
 
-  data->old_allocation = *allocation;
-
 }
 
 static void
@@ -1295,6 +1198,9 @@ hildon_home_area_child_build_alpha_mask (HildonHomeArea *area,
                         borders->left + center_w, borders->top + center_h,
                         borders->right, borders->bottom);
 
+      child_data->mask_width = w;
+      child_data->mask_height = h;
+
       XFreePixmap (GDK_DISPLAY (), p);
       g_free (borders);
     }
@@ -1314,6 +1220,12 @@ hildon_home_area_child_state_change (ChildData *child_data)
 
   child_data->state = state;
 
+  if (state == 0 && /* not moving or resizing */
+      (child_data->mask_width  != child_data->widget->allocation.width ||
+       child_data->mask_height != child_data->widget->allocation.height))
+    hildon_home_area_child_build_alpha_mask (HILDON_HOME_AREA (child_data->widget->parent),
+                                             child_data->widget);
+
 }
 
 
@@ -1531,8 +1443,8 @@ hildon_home_area_sort_stack (HildonHomeArea *area)
   for (child = priv->children_data; child ; child = child->next)
     {
       ChildData        *data = child->data;
-      if (GTK_WIDGET_REALIZED (data->widget))
-        gdk_window_raise (data->widget->window);
+      if (HILDON_DESKTOP_IS_HOME_ITEM (data->widget))
+        hildon_desktop_home_item_raise (HILDON_DESKTOP_HOME_ITEM (data->widget));
     }
 }
 
@@ -1695,13 +1607,11 @@ remove_widget (GtkWidget *widget, GList *region)
   GdkRectangle r = {0};
   gint x, y;
   gint padding;
-  const gchar *name;
 
   gtk_container_child_get (GTK_CONTAINER (widget->parent), widget,
                            "x", &x,
                            "y", &y,
                            NULL);
-  g_object_get (widget, "id", &name, NULL);
 
   g_object_get (widget->parent,
                 "applet-padding", &padding,
@@ -1840,6 +1750,7 @@ hildon_home_area_batch_add (HildonHomeArea *area)
   g_list_foreach (region, (GFunc)g_free, NULL);
   g_list_free (region);
 
+  g_list_free (priv->to_add);
   priv->to_add = NULL;
 
 }
@@ -1853,11 +1764,6 @@ hildon_home_area_remove (GtkContainer *area, GtkWidget *applet)
   g_return_if_fail (area);
 
   priv = HILDON_HOME_AREA_GET_PRIVATE (area);
-  if (priv->layout_mode)
-    {
-      g_signal_emit_by_name (area, "layout-changed");
-    }
-
   gtk_container_child_get (area, applet,
                            "child-data", &child_data,
                            NULL);
@@ -1893,39 +1799,6 @@ hildon_home_area_new ()
   return g_object_new (HILDON_TYPE_HOME_AREA, NULL);
 }
 
-void
-hildon_home_area_set_layout_mode (HildonHomeArea *area, gboolean layout_mode)
-{
-  HildonHomeAreaPriv      *priv;
-  g_return_if_fail (area);
-
-  priv = HILDON_HOME_AREA_GET_PRIVATE (area);
-
-  if (priv->layout_mode != layout_mode)
-    {
-      g_object_notify (G_OBJECT (area), "layout-mode");
-      priv->layout_mode = layout_mode;
-
-      if (priv->layout_mode)
-        g_signal_emit_by_name (G_OBJECT (area), "layout-mode-start");
-      else
-        g_signal_emit_by_name (G_OBJECT (area), "layout-mode-end");
-
-    }
-}
-
-gboolean
-hildon_home_area_get_layout_mode (HildonHomeArea *area)
-{
-  HildonHomeAreaPriv      *priv;
-
-  g_return_val_if_fail (area, FALSE);
-
-  priv = HILDON_HOME_AREA_GET_PRIVATE (area);
-
-  return priv->layout_mode;
-}
-
 static void
 hildon_home_area_child_save_position (ChildData *child, GKeyFile *keyfile)
 {
@@ -2053,11 +1926,9 @@ hildon_home_area_save_configuration (HildonHomeArea *area,
     }
 
   else
-    {
-      ret = fwrite (buffer, buffer_size, 1, file);
+    ret = fwrite (buffer, buffer_size, 1, file);
 
-      g_free (buffer);
-    }
+  g_free (buffer);
 
   if (file)
     fclose (file);
@@ -2203,32 +2074,6 @@ hildon_home_area_get_layout_changed (HildonHomeArea *area)
   return priv->layout_changed;
 }
 
-gboolean
-hildon_home_area_get_overlaps (HildonHomeArea *area)
-{
-  gboolean overlap;
-  GList *applets, *l;
-
-  g_return_val_if_fail (area, FALSE);
-
-  overlap = FALSE;
-
-  applets = gtk_container_get_children (GTK_CONTAINER (area));
-
-  for (l = applets; l && !overlap; l = g_list_next (l))
-    {
-      if (!HILDON_DESKTOP_IS_HOME_ITEM (l->data))
-        continue;
-
-      overlap = hildon_desktop_home_item_get_overlaps
-          (HILDON_DESKTOP_HOME_ITEM (l->data));
-    }
-
-  g_list_free (applets);
-
-  return overlap;
-}
-
 void
 hildon_home_area_set_batch_add (HildonHomeArea *area, gboolean batch_add)
 {
