@@ -196,7 +196,7 @@ hd_keys_action_power (HDKeysConfig *keys,
   DBusError             error;
   DBusMessage          *msg = NULL;
   gboolean              long_press = GPOINTER_TO_INT(user_data);
-  
+
   dbus_error_init (&error);
   connection = dbus_bus_get (DBUS_BUS_SYSTEM, &error);
 
@@ -216,7 +216,7 @@ hd_keys_action_power (HDKeysConfig *keys,
 
   dbus_message_append_args (msg,
                             DBUS_TYPE_BOOLEAN,
-                            long_press,
+                            &long_press,
                             DBUS_TYPE_INVALID);
 
   dbus_connection_send (connection, msg, NULL);
