@@ -300,7 +300,7 @@ hd_wm_atoms_init (HDWM *hdwm)
     "_MB_COMMAND",              /* FIXME: Unused */
     "_MB_CURRENT_APP_WINDOW",
     "_MB_APP_WINDOW_LIST_STACKING",
-    "_MB_NUM_MODAL_WINDOWS_PRESENT",
+    "_MB_NUM_SYSTEM_MODAL_WINDOWS_PRESENT",
     
     "UTF8_STRING",
 
@@ -561,6 +561,41 @@ hd_wm_dbus_signal_handler (DBusConnection *conn, DBusMessage *msg, void *data)
 {
   HDWM *hdwm = HD_WM (data);
 
+  if (dbus_message_is_method_call (msg,
+                                   HDWM_STARTUP_NOTIFICATION_IFACE,
+                                   HDWM_STARTUP_NOTIFICATION_STARTING))
+  {
+    HDWMApplication    *app;
+    gchar              *service;
+    DBusError           err;
+
+    dbus_error_init(&err);
+    dbus_message_get_args(msg, &err,
+			  DBUS_TYPE_STRING, &service,
+			  DBUS_TYPE_INVALID);
+
+    if (dbus_error_is_set (&err))
+    {
+	g_warning ("Error getting message args: %s\n",
+                   err.message);
+        dbus_error_free (&err);
+	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+    }
+
+    g_debug ("application starting: %s", service);
+
+    app = hd_wm_lookup_application_via_service (service);
+
+    if (app)
+    {
+      g_signal_emit_by_name (hdwm,
+                             "application-starting",
+                             app);
+    }
+
+    return DBUS_HANDLER_RESULT_HANDLED;
+
+  }
   if (dbus_message_is_signal(msg, MAEMO_LAUNCHER_SIGNAL_IFACE,
 				  APP_DIED_SIGNAL_NAME))
   {
@@ -785,6 +820,11 @@ hd_wm_constructor (GType gtype, guint n_params, GObjectConstructParam *params)
     dbus_bus_add_match (connection, "type='signal', interface='"
                         MAEMO_LAUNCHER_SIGNAL_IFACE "'", NULL);
 
+    dbus_bus_add_match (connection,
+                        "type='method_call', "
+                        "interface='"HDWM_STARTUP_NOTIFICATION_IFACE"'",
+                        NULL);
+
     dbus_connection_add_filter (connection, hd_wm_dbus_signal_handler,
                                 hdwm, NULL);
 
@@ -1203,7 +1243,6 @@ hd_wm_top_item (HDWMEntryInfo *info)
     XSync (GDK_DISPLAY(),FALSE);
     
     gdk_error_trap_pop();
-
   }
 
   if (HD_WM_IS_DESKTOP (info))
@@ -1279,7 +1318,7 @@ hd_wm_activate_service (const gchar *app, const gchar *parameters)
   {
     if ((wapp = hd_wm_lookup_application_via_service (service)) != NULL)
     {
-      if (hd_wm_application_has_startup_notify (wapp) && 
+      if (hd_wm_application_has_startup_notify (wapp) &&
 	  hdwm->priv->lowmem_banner_timeout >= 0 &&
 	  !hd_wm_application_has_windows (wapp))
       { 
@@ -1630,7 +1669,6 @@ hd_wm_process_mb_current_app_window (HDWM *hdwm)
   HDWMWindow *win;
   Window            *app_xwin;
 
-  
   if(hdwm->priv->active_window)
     previous_app_xwin = hd_wm_window_get_x_win (hdwm->priv->active_window);
   
@@ -1732,7 +1770,6 @@ client_list_steal_foreach_func (gpointer key,
       if (app)
         app_info = HD_WM_ENTRY_INFO (app);
       
-
       g_signal_emit_by_name (hdwm,"entry_info_changed",app_info);
       /* free the original hash key, since we are stealing */
       g_free (key);
@@ -1919,7 +1956,7 @@ hd_wm_process_x_client_list (HDWM *hdwm)
 {
   struct xwinv xwins;
   int     i;
- 
+  
   /* Try to get desktop home window the first this function is called 
    * This is f*ck*ng expensive but it is supposed to work at the startup and 
    * you should forget about it 
@@ -1959,7 +1996,11 @@ hd_wm_process_x_client_list (HDWM *hdwm)
 	hd_wm_desktop_set_x_window (HD_WM_DESKTOP (hd_wm_get_home_info (hdwm)), allwins.wins[i]);
 	break;
       }	      
+
+      XFree (wm_type_atom);
     }
+
+    XFree (allwins.wins);
   }
 
   /* FIXME: We (or MB!) should probably keep a copy of ordered window list
@@ -2043,7 +2084,8 @@ hd_wm_process_x_client_list (HDWM *hdwm)
     
       /* if the window does not have attached info yet, then it is new
        * and needs to be added to AS; if it has one, then it is coming
-       * out of hibernation, in which case it must not be added
+       * out of hibernation, in which case it must not be added but the
+       * stack change is notified
        */
       info = HD_WM_ENTRY_INFO (win);
 	   
@@ -2051,8 +2093,12 @@ hd_wm_process_x_client_list (HDWM *hdwm)
       {
         hd_wm_debug ("Adding AS entry for view-less window\n");
         hd_wm_add_applications (hdwm,info);
-        g_signal_emit_by_name (hdwm,"entry_info_added",info);		
-       }
+        g_signal_emit_by_name (hdwm,"entry_info_added",info);
+      }
+      else
+      {
+        g_signal_emit_by_name (hdwm,"entry_info_stack_changed",info);
+      }
     }
   }
 
