@@ -101,6 +101,10 @@
 #define APPKILLER_SIGNAL_PATH      "/com/nokia/osso_app_killer"
 #define APPKILLER_SIGNAL_NAME      "exit"
 
+#define HDWM_STARTUP_NOTIFICATION_IFACE "com.nokia.hildon.hdwm.startupnotification"
+#define HDWM_OBJECT_PATH                "/com/nokia/hildon/hdwm"
+#define HDWM_STARTUP_NOTIFICATION_STARTING  "starting"
+
 #define SAVE_METHOD      "save"
 #define KILL_APPS_METHOD "kill_app"
 
