@@ -39,6 +39,7 @@
 
 #define HD_APP_MENU_DIALOG_WIDTH        590
 #define HD_APP_MENU_DIALOG_HEIGHT       (8*30 + 2*HILDON_MARGIN_DEFAULT)
+#define HD_APP_MENU_DIALOG_NEW_WIDTH    300
 
 enum
 {
@@ -315,6 +316,8 @@ hd_app_menu_dialog_new_category (HDAppMenuDialog *dialog)
                                               GTK_DIALOG_DESTROY_WITH_PARENT,
                                             NULL);
 
+  gtk_widget_set_size_request (new_dialog, HD_APP_MENU_DIALOG_NEW_WIDTH, -1);
+
   ok_button = gtk_dialog_add_button (GTK_DIALOG (new_dialog),
                                      HD_APP_MENU_DIALOG_NEW_OK,
                                      GTK_RESPONSE_OK);
@@ -418,6 +421,8 @@ hd_app_menu_dialog_rename_category (HDAppMenuDialog *dialog)
                                               GTK_DIALOG_DESTROY_WITH_PARENT,
                                               NULL);
 
+  gtk_widget_set_size_request (new_dialog, HD_APP_MENU_DIALOG_NEW_WIDTH, -1);
+
   ok_button = gtk_dialog_add_button (GTK_DIALOG (new_dialog),
                                      HD_APP_MENU_DIALOG_RENAME_OK,
                                      GTK_RESPONSE_OK);
