@@ -106,11 +106,13 @@ struct _HDApplicationsMenuPrivate
   GnomeVFSMonitorHandle    *desktop_dir_monitor;
   guint                     monitor_update_timeout;
   gboolean                  focus_applications;
+  gboolean                  motion_reset;
 };
 
 static void hd_applications_menu_register_monitors (HDApplicationsMenu *button);
 static void hd_applications_menu_create_menu (HDApplicationsMenu *button);
 static void hd_applications_menu_button_toggled (GtkWidget *widget, HDApplicationsMenu *button);
+static gboolean hd_applications_menu_button_button_press (GtkWidget *widget, GdkEventButton *event, HDApplicationsMenu *button);
 static gboolean hd_applications_menu_button_key_press (GtkWidget* widget, GdkEventKey *event, HDApplicationsMenu *button);
 
 static void
@@ -159,9 +161,12 @@ hd_applications_menu_init (HDApplicationsMenu *button)
 
   button->priv = priv;
 
-  button->priv->system_dir_monitor = NULL;
-  button->priv->home_dir_monitor = NULL;
-  button->priv->desktop_dir_monitor = NULL;
+  priv->system_dir_monitor = NULL;
+  priv->home_dir_monitor = NULL;
+  priv->desktop_dir_monitor = NULL;
+
+  priv->focus_applications = FALSE;
+  priv->motion_reset = FALSE;
   
   priv->button = hildon_desktop_toggle_button_new ();
 
@@ -186,6 +191,16 @@ hd_applications_menu_init (HDApplicationsMenu *button)
       	            G_CALLBACK (hd_applications_menu_button_toggled), 
 		    button);
 
+   g_signal_connect (G_OBJECT (priv->button), 
+		    "button-press-event",
+      	            G_CALLBACK (hd_applications_menu_button_button_press), 
+		    button);
+
+  g_signal_connect (G_OBJECT (priv->button), 
+		    "button-release-event",
+      	            G_CALLBACK (gtk_true), 
+		    NULL);
+
   g_signal_connect (G_OBJECT (priv->button), 
 		    "key-press-event",
  		     G_CALLBACK (hd_applications_menu_button_key_press), 
@@ -193,8 +208,6 @@ hd_applications_menu_init (HDApplicationsMenu *button)
   
   priv->monitor_update_timeout = 0;
 
-  priv->focus_applications = TRUE;
-  
   priv->popup_window = NULL;
   
   gtk_container_add (GTK_CONTAINER (button), priv->button);
@@ -237,8 +250,6 @@ hd_applications_menu_popdown (GtkWidget *menu, HDApplicationsMenu *button)
 
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button->priv->button),
 			        FALSE);
-
-  button->priv->focus_applications = TRUE;
 }
 
 static gboolean
@@ -246,10 +257,29 @@ hd_applications_menu_key_press (GtkWidget *menu,
 			        GdkEventKey *event,
 			        HDApplicationsMenu *button)
 {
+  gboolean activate_menu = FALSE, cancel_menu = FALSE;
+  
   HDWM *hdwm = hd_wm_get_singleton ();
 
-  if (event->keyval == GDK_Left    ||
-      event->keyval == GDK_KP_Left ||
+  if (gtk_widget_get_direction (menu) == GTK_TEXT_DIR_RTL)
+  {
+    if (event->keyval == GDK_Left ||
+        event->keyval == GDK_KP_Left)
+      activate_menu = TRUE;
+    else if (event->keyval == GDK_Right || 
+             event->keyval == GDK_KP_Right)
+      cancel_menu = TRUE;
+  } 
+  else 
+  {
+    if (event->keyval == GDK_Right)
+      activate_menu = TRUE;
+    else if (event->keyval == GDK_Left || 
+             event->keyval == GDK_KP_Left)
+      cancel_menu = TRUE;
+  }  
+
+  if (cancel_menu ||
       event->keyval == GDK_Escape)
   {
     if (hd_applications_menu_has_focus (button->priv->menu_applications))
@@ -259,12 +289,13 @@ hd_applications_menu_key_press (GtkWidget *menu,
 
       hildon_desktop_popup_menu_deselect_item (button->priv->menu_applications,
 		      			       selected_item);
-      
+ 
       selected_item = 
         hildon_desktop_popup_menu_get_selected_item (button->priv->menu_categories);
 
       gtk_widget_grab_focus (GTK_WIDGET (selected_item));
-      
+      gtk_widget_queue_draw (GTK_WIDGET (selected_item));
+ 
       return TRUE;
     }
   
@@ -290,26 +321,13 @@ hd_applications_menu_key_press (GtkWidget *menu,
   }
   else if (event->keyval == GDK_Return    ||
 	   event->keyval == GDK_KP_Enter  ||
-	   event->keyval == GDK_ISO_Enter ||
-	   event->keyval == GDK_Right     ||
-           event->keyval == GDK_KP_Right)
+	   event->keyval == GDK_ISO_Enter)
   {
     if (hd_applications_menu_has_focus (button->priv->menu_categories))
     {
-      GtkMenuItem *selected_item;
-
-      hildon_desktop_popup_menu_select_first_item (button->priv->menu_applications);
-
-      selected_item = 
-        hildon_desktop_popup_menu_get_selected_item (button->priv->menu_applications);
-
-      if (GTK_WIDGET_IS_SENSITIVE (selected_item))
-        gtk_widget_grab_focus (GTK_WIDGET (selected_item));
-      else
-	hildon_desktop_popup_menu_deselect_item (button->priv->menu_applications,
-			                         selected_item);
+      button->priv->focus_applications = TRUE;
       
-      return TRUE;
+      return gtk_widget_event (GTK_WIDGET (button->priv->menu_categories), (GdkEvent *) event);
     }
     else if (hd_applications_menu_has_focus (button->priv->menu_applications))
     {
@@ -322,16 +340,18 @@ hd_applications_menu_key_press (GtkWidget *menu,
     {
       gtk_widget_event (GTK_WIDGET (button->priv->menu_categories), (GdkEvent *) event);
 
-      if (event->keyval == GDK_Up     ||
-          event->keyval == GDK_KP_Up  ||
-          event->keyval == GDK_Down   ||
-          event->keyval == GDK_KP_Down)
+      if (event->keyval == GDK_Up      ||
+          event->keyval == GDK_KP_Up   ||
+          event->keyval == GDK_Down    ||
+          event->keyval == GDK_KP_Down ||
+	  activate_menu)
       {
         GtkMenuItem *selected_item;
 
 	selected_item = 
           hildon_desktop_popup_menu_get_selected_item (button->priv->menu_categories);
 
+	button->priv->focus_applications = activate_menu;
 	gtk_menu_item_activate (selected_item);
       }
     }
@@ -344,47 +364,42 @@ hd_applications_menu_key_press (GtkWidget *menu,
   return FALSE;
 }
 
-static gboolean 
-hd_applications_menu_button_release_category (GtkMenuItem *item, 
-					    GdkEventButton *event,
-					    HDApplicationsMenu *button)
-{
-  button->priv->focus_applications = TRUE;
-
-  gtk_menu_item_activate (item);
-
-  return TRUE;
-}
-
 static void
 hd_applications_menu_activate_category (GtkMenuItem *item, HDApplicationsMenu *button)
 {
-  GList *sub_items, *i;
+  static GtkMenuItem *last_selected_item = NULL;
 
+  GList *sub_items, *i;
+  
   sub_items = hildon_desktop_popup_menu_get_children 
 	  (button->priv->menu_applications);
   
-  for (i = g_list_first (sub_items); i; i = i->next)
+  if (last_selected_item != item)
   {
-    GtkMenuItem *child = (GtkMenuItem *) i->data;
+    last_selected_item = item;
+	  
+    for (i = g_list_first (sub_items); i; i = i->next)
+    {
+      GtkMenuItem *child = (GtkMenuItem *) i->data;
 
-    g_object_ref (child);
+      g_object_ref (child);
 
-    hildon_desktop_popup_menu_remove_item
-      (button->priv->menu_applications, child);
-  }
+      hildon_desktop_popup_menu_remove_item
+        (button->priv->menu_applications, child);
+    }
 
-  g_list_free (sub_items);
+    g_list_free (sub_items);
 
-  sub_items = (GList *) g_object_get_data (G_OBJECT (item),
-		                           CATEGORY_SUB_ITEMS);
+    sub_items = (GList *) g_object_get_data (G_OBJECT (item),
+          	                           CATEGORY_SUB_ITEMS);
 
-  for (i = g_list_first (sub_items); i; i = i->next)
-  {
-    GtkMenuItem *child = (GtkMenuItem *) i->data;
+    for (i = g_list_first (sub_items); i; i = i->next)
+    {
+      GtkMenuItem *child = (GtkMenuItem *) i->data;
 
-    hildon_desktop_popup_menu_add_item
-      (button->priv->menu_applications, child);
+      hildon_desktop_popup_menu_add_item
+        (button->priv->menu_applications, child);
+    }
   }
 
   if (button->priv->focus_applications &&
@@ -392,6 +407,7 @@ hd_applications_menu_activate_category (GtkMenuItem *item, HDApplicationsMenu *b
   {
     hildon_desktop_popup_menu_select_first_item (button->priv->menu_applications);
     gtk_widget_grab_focus (GTK_WIDGET (sub_items->data));
+    gtk_widget_queue_draw (GTK_WIDGET (item));
   }
   else
   {
@@ -623,7 +639,8 @@ hd_applications_menu_get_items (HDApplicationsMenu *button,
       hbox = gtk_hbox_new (FALSE, 0);
       
       label = gtk_label_new (item_name);
-      
+     
+      gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
       gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
       
       gtk_box_pack_start (GTK_BOX (hbox),
@@ -631,7 +648,7 @@ hd_applications_menu_get_items (HDApplicationsMenu *button,
       		          TRUE, 
       		          TRUE, 
       		          6);
-      
+
       gtk_container_add (GTK_CONTAINER (menu_item), hbox);
       
       gtk_widget_show_all (hbox);
@@ -662,11 +679,6 @@ hd_applications_menu_get_items (HDApplicationsMenu *button,
       		              sub_items, 
 			      (GDestroyNotify) hd_applications_menu_free_menu_items);
 
-      g_signal_connect (G_OBJECT (menu_item), 
-		        "button-release-event",
-      		        G_CALLBACK (hd_applications_menu_button_release_category),
-      		        button);
-
       g_signal_connect (G_OBJECT (menu_item), 
 		        "activate",
       		        G_CALLBACK (hd_applications_menu_activate_category),
@@ -674,9 +686,17 @@ hd_applications_menu_get_items (HDApplicationsMenu *button,
     }
     else if (g_str_equal (item_desktop_id, SEPARATOR_STRING))
     {
+	GtkRequisition req;
+	    
       /* Separator */
       menu_item = gtk_separator_menu_item_new ();
-      gtk_widget_set_size_request (GTK_WIDGET (menu_item), -1, 1);
+
+      if (level > 0)
+        gtk_widget_set_name (GTK_WIDGET (menu_item), HD_APPS_MENU_APP_MENU_ITEM_NAME);
+      else
+        gtk_widget_set_name (GTK_WIDGET (menu_item), HD_APPS_MENU_CAT_MENU_ITEM_NAME);
+ 
+      gtk_widget_size_request (GTK_WIDGET (menu_item), &req);
     }
     else if (level > 0)
     {
@@ -724,6 +744,7 @@ hd_applications_menu_get_items (HDApplicationsMenu *button,
         label = gtk_label_new ((item_text_domain && *item_text_domain) ?
 			        dgettext(item_text_domain, item_name) : _(item_name)), 
 
+        gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
         gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	
 	gtk_box_pack_start (GTK_BOX (hbox),
@@ -759,8 +780,11 @@ hd_applications_menu_get_items (HDApplicationsMenu *button,
       if (!first_item)
       {
         GtkWidget *separator = gtk_separator_menu_item_new ();
-
-	gtk_widget_set_size_request (GTK_WIDGET (separator), -1, 1);
+	
+        if (level > 0)
+          gtk_widget_set_name (GTK_WIDGET (separator), HD_APPS_MENU_APP_MENU_ITEM_NAME);
+        else
+          gtk_widget_set_name (GTK_WIDGET (separator), HD_APPS_MENU_CAT_MENU_ITEM_NAME);
 
         menu_items = g_list_prepend (menu_items, separator);
       }
@@ -809,6 +833,19 @@ hd_applications_menu_populate (HDApplicationsMenu *button)
   g_list_free (menu_items);
 }
 
+static gboolean 
+hd_applications_menu_button_button_press (GtkWidget *widget, 
+					  GdkEventButton *event,
+					  HDApplicationsMenu *button)
+{
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), 
+		  		!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)));
+
+  gtk_toggle_button_toggled (GTK_TOGGLE_BUTTON (widget));
+  
+  return TRUE;
+}
+
 static gboolean 
 hd_applications_menu_categories_motion_notify (GtkWidget      *widget,
 					       GdkEventMotion *event,
@@ -819,12 +856,15 @@ hd_applications_menu_categories_motion_notify (GtkWidget      *widget,
   GtkMenuItem *menu_item = 
     hildon_desktop_popup_menu_get_selected_item (HILDON_DESKTOP_POPUP_MENU (widget));
 
-  if (menu_item != selected_item)
+  if (menu_item != selected_item ||
+      button->priv->motion_reset)
   {
     selected_item = menu_item;
 
     button->priv->focus_applications = TRUE;
     gtk_menu_item_activate (selected_item);
+
+    button->priv->motion_reset = FALSE;
   }
   
   return TRUE;
@@ -858,8 +898,10 @@ hd_applications_menu_create_menu (HDApplicationsMenu *button)
   /* We don't attach the widget because if we do it, we cannot be on top of 
    * virtual keyboard. Anyway it should be transient to button->priv->button
    */
+#if 0
   hildon_desktop_popup_window_attach_widget (popup_window, NULL);
-  
+#endif
+
   button->priv->menu_categories =
     HILDON_DESKTOP_POPUP_MENU (g_object_new (HILDON_DESKTOP_TYPE_POPUP_MENU,
 		  		    	     "item-height", 72,
@@ -937,10 +979,10 @@ hd_applications_menu_create_menu (HDApplicationsMenu *button)
   gtk_container_add (GTK_CONTAINER (button_down), arrow);
   gtk_widget_show (arrow);
 
-  g_signal_connect (G_OBJECT (popup_window), 
-		    "key-press-event",
-		    G_CALLBACK (hd_applications_menu_key_press),
-		    button);
+   g_signal_connect (G_OBJECT (popup_window),
+                     "key-press-event",
+                     G_CALLBACK (hd_applications_menu_key_press),
+                     button);
   
   g_signal_connect (G_OBJECT (popup_window), 
 		    "popdown-window",
@@ -1096,6 +1138,8 @@ hd_applications_menu_show (HDApplicationsMenu * button)
   if (!button->priv->popup_window)
     hd_applications_menu_create_menu (button);
 
+  button->priv->motion_reset = TRUE;
+  
   hildon_desktop_popup_window_popup (button->priv->popup_window,
 		                     (HDPopupWindowPositionFunc) hd_applications_menu_get_menu_position,
 		                     button,
@@ -1108,6 +1152,7 @@ hd_applications_menu_show (HDApplicationsMenu * button)
 
   if (menu_items != NULL)
   {
+    button->priv->focus_applications = TRUE;
     gtk_menu_item_activate (GTK_MENU_ITEM (menu_items->data));
   }
 
@@ -1119,7 +1164,16 @@ hd_applications_menu_button_toggled (GtkWidget *widget, HDApplicationsMenu *butt
 {
   if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
   {
+    GtkMenuItem *selected_item;
+
+    selected_item = 
+      hildon_desktop_popup_menu_get_selected_item (button->priv->menu_applications);
+
+    hildon_desktop_popup_menu_deselect_item (button->priv->menu_applications,
+		    			     selected_item);
+
     hildon_desktop_popup_window_popdown (button->priv->popup_window);
+
     return;
   } 
 
@@ -1131,17 +1185,37 @@ hd_applications_menu_button_key_press (GtkWidget *widget,
 		                       GdkEventKey *event,
 			               HDApplicationsMenu *button)
 {
+  gboolean activate_button = FALSE, cancel_button = FALSE;
+  
   g_return_val_if_fail (button, FALSE);
 
-  if (event->keyval == GDK_Right ||
-      event->keyval == GDK_KP_Enter)
+  if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
+  {
+    if (event->keyval == GDK_Left ||
+        event->keyval == GDK_KP_Left ||
+        event->keyval == GDK_KP_Enter)
+      activate_button = TRUE;
+    else if (event->keyval == GDK_Right || 
+             event->keyval == GDK_KP_Right)
+      cancel_button = TRUE;
+  } 
+  else 
+  {
+    if (event->keyval == GDK_Right ||
+        event->keyval == GDK_KP_Enter)
+      activate_button = TRUE;
+    else if (event->keyval == GDK_Left || 
+             event->keyval == GDK_KP_Left)
+      cancel_button = TRUE;
+  }
+
+  if (activate_button)
   {
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button->priv->button), TRUE); 
 
     return TRUE;
   }
-  else if (event->keyval == GDK_Left || 
-           event->keyval == GDK_KP_Left)
+  else if (cancel_button)
   {
     gtk_widget_grab_focus (widget);
 	  
@@ -1165,6 +1239,9 @@ hd_applications_menu_changed (HDApplicationsMenu *button)
 
   button->priv->monitor_update_timeout = 0;
 
+  if (button->priv->popup_window)
+    hildon_desktop_popup_window_popdown (button->priv->popup_window);
+  
   hd_applications_menu_create_menu (button);
   
   return FALSE;
