@@ -40,6 +40,7 @@
 
 #include <libhildonwm/hd-wm.h>
 #include <libhildondesktop/hildon-desktop-window.h>
+#include <libhildondesktop/hildon-desktop-container.h>
 #include <libhildondesktop/hildon-desktop-notification-manager.h>
 
 #include <hildon/hildon-banner.h>
@@ -91,6 +92,7 @@ typedef struct
   gboolean                is_ordered;
   gboolean                load_new_plugins;
   GnomeVFSMonitorHandle  *plugin_dir_monitor;
+  gboolean                ignore_next_monitor;
 } HDDesktopContainerInfo;
 
 typedef struct
@@ -123,21 +125,23 @@ void
 hd_desktop_launch_banner_close (GtkWidget *parent, HDDesktopBannerInfo *info);
 
 static void
-hd_desktop_ping_timeout_dialog_response (GtkDialog *note, gint ret, gpointer data)
+hd_desktop_ping_timeout_dialog_response (GtkDialog *note,
+                                         gint ret,
+                                         gpointer data)
 {
   HDWMWindow *win = (HDWMWindow *)data;
 
-  /* This is for NB#64333: If the application recover once the dialog 
-   * has been shown we end up having the situation where we try to kill an 
+  /* This is for NB#64333: If the application recover once the dialog
+   * has been shown we end up having the situation where we try to kill an
    * application that has already been gone, hence destroyed.
    */
 
   if (!HD_WM_IS_WINDOW (win))
-  {	  
+  {
     gtk_widget_destroy (GTK_WIDGET(note));
     return;
   }
-	  
+
   HDWMApplication *app = hd_wm_window_get_application (win);
 
   gtk_widget_destroy (GTK_WIDGET(note));
@@ -145,35 +149,43 @@ hd_desktop_ping_timeout_dialog_response (GtkDialog *note, gint ret, gpointer dat
 
   if (ret == GTK_RESPONSE_OK)
   {
-    /* Kill the app */	  
+    /* Kill the app */
     if (!hd_wm_window_attempt_signal_kill (win, SIGKILL, FALSE))
       g_debug ("hd_wm_ping_timeout:failed to kill application '%s'.", hd_wm_window_get_name (win));
   }
 }
 
+static void
+destroy_note (HDWMApplication *app)
+{
+  GtkWidget *note;
+  note = GTK_WIDGET (hd_wm_application_get_ping_timeout_note (app));
+  gtk_widget_destroy (note);
+  hd_wm_application_set_ping_timeout_note (app, NULL);
+}
+
 static void
 hd_desktop_application_frozen (HDWM *hdwm, HDWMWindow *win, gpointer data)
 {
   GtkWidget *note;
-  g_debug ("application frozen");
 
   HDWMApplication *app = hd_wm_window_get_application (win);
 
-  gchar *timeout_message = 
+  gchar *timeout_message =
     g_strdup_printf (HD_DESKTOP_PING_TIMEOUT_MESSAGE_STRING, hd_wm_window_get_name (win));
 
   /* FIXME: Do we need to check if the note already exists? */
   note = GTK_WIDGET (hd_wm_application_get_ping_timeout_note (app));
-  
+
   if (note && GTK_IS_WIDGET (note))
   {
-    g_debug ("hd_wm_ping_timeout: the note already exists.");
     goto cleanup_and_exit;
   }
 
   note = hildon_note_new_confirmation (NULL, timeout_message);
 
   hd_wm_application_set_ping_timeout_note (app, G_OBJECT (note));
+  g_object_weak_ref (G_OBJECT (win), (GWeakNotify)destroy_note, app);
 
   hildon_note_set_button_texts (HILDON_NOTE (note),
                                 HD_DESKTOP_PING_TIMEOUT_BUTTON_OK_STRING,
@@ -310,9 +322,10 @@ static gchar *
 hd_desktop_get_conf_file_path (const gchar *config_file)
 {
   gchar *config_file_path;
-        
+  gchar *user_config_file_path;
+
   config_file_path = g_build_filename (g_get_home_dir (),
-                                       HD_DESKTOP_CONFIG_USER_PATH, 
+                                       HD_DESKTOP_CONFIG_USER_PATH,
                                        config_file,
                                        NULL);
 
@@ -320,7 +333,7 @@ hd_desktop_get_conf_file_path (const gchar *config_file)
      configuration file at sysconfdir. */
   if (!g_file_test (config_file_path, G_FILE_TEST_EXISTS))
   {
-    g_free (config_file_path);
+    user_config_file_path = config_file_path;
 
     config_file_path = g_build_filename (HD_DESKTOP_CONFIG_PATH, 
                                          config_file,
@@ -331,9 +344,29 @@ hd_desktop_get_conf_file_path (const gchar *config_file)
       g_free (config_file_path);
       config_file_path = NULL;
     }
+    else
+    {
+      /* If system-wide configuration file is present, but not
+       * user-wide, copy it the system-wide */
+      gchar *content;
+      gsize length;
+
+      if (g_file_get_contents (config_file_path,
+                               &content,
+                               &length,
+                               NULL))
+      {
+        g_file_set_contents (user_config_file_path,
+                             content,
+                             length,
+                             NULL);
+      }
+
+    }
+    g_free (user_config_file_path);
   }
 
-  return config_file_path; 
+  return config_file_path;
 }
 
 static GList *
@@ -524,13 +557,14 @@ hd_desktop_container_save (HildonDesktopWindow *window, gpointer user_data)
 
   plugin_list = hd_desktop_plugin_list_from_container (window->container);
 
+  info->ignore_next_monitor = TRUE;
   hd_desktop_plugin_list_to_conf (plugin_list, info->config_file);
 
   g_list_foreach (plugin_list, (GFunc) g_free , NULL);
   g_list_free (plugin_list); 
 }
 
-static void 
+static void
 hd_desktop_select_plugins (HildonDesktopWindow *window, gpointer user_data)
 {
   HDDesktopPrivate *priv;
@@ -542,7 +576,7 @@ hd_desktop_select_plugins (HildonDesktopWindow *window, gpointer user_data)
   g_return_if_fail (user_data != NULL);
 
   info = (HDDesktopContainerInfo *) user_data;
-  
+
   g_return_if_fail (HD_IS_DESKTOP (info->desktop));
 
   priv = info->desktop->priv;
@@ -563,6 +597,8 @@ hd_desktop_select_plugins (HildonDesktopWindow *window, gpointer user_data)
                             window->container,
 			    info->policy,
 			    info->is_ordered);
+
+    hd_desktop_container_save (window, info);
   }
 
   g_list_foreach (selected_plugins, (GFunc) g_free , NULL);
@@ -635,6 +671,15 @@ hd_desktop_system_conf_dir_changed (GnomeVFSMonitorHandle *handle,
     if (info != NULL)
     {
       gchar *config_file;
+
+      /* If the ignore_next_monitor flag is set, we ignore this event.
+       * The flag is set when the container configuration is saved */
+      if (info->ignore_next_monitor)
+      {
+        info->ignore_next_monitor = FALSE;
+        g_free (filename);
+        return;
+      }
       
       config_file = hd_desktop_get_conf_file_path (info->config_file);
       
@@ -690,6 +735,15 @@ hd_desktop_user_conf_dir_changed (GnomeVFSMonitorHandle *handle,
     if (info != NULL)
     {
       gchar *config_file;
+
+      /* If the ignore_next_monitor flag is set, we ignore this event.
+       * The flag is set when the container configuration is saved */
+      if (info->ignore_next_monitor)
+      {
+        info->ignore_next_monitor = FALSE;
+        g_free (filename);
+        return;
+      }
       
       config_file = hd_desktop_get_conf_file_path (info->config_file);
       
@@ -719,6 +773,7 @@ hd_desktop_plugin_dir_changed (GnomeVFSMonitorHandle *handle,
 {
   GList *plugin_list = NULL, *children, *iter;
   gboolean update = FALSE;
+  GtkContainer *container;
 
   if (event_type != GNOME_VFS_MONITOR_EVENT_DELETED &&
       event_type != GNOME_VFS_MONITOR_EVENT_CREATED)
@@ -737,9 +792,13 @@ hd_desktop_plugin_dir_changed (GnomeVFSMonitorHandle *handle,
 
     plugin_list = g_list_append (plugin_list, uri_str);
   }
-    
-  children = gtk_container_get_children (
-                  HILDON_DESKTOP_WINDOW (info->container)->container);
+
+  container = HILDON_DESKTOP_WINDOW (info->container)->container;
+
+  if (HILDON_DESKTOP_IS_CONTAINER (container))
+    children = hildon_desktop_container_get_children (HILDON_DESKTOP_CONTAINER (container));
+  else
+    children = gtk_container_get_children (container);
 
   for (iter = children; iter; iter = g_list_next (iter))
   {
@@ -904,7 +963,7 @@ hd_desktop_load_containers (HDDesktop *desktop)
       continue;
     }
 
-    container_config = hd_desktop_get_conf_file_path (container_config_file); 
+    container_config = hd_desktop_get_conf_file_path (container_config_file);
 
     if (container_config == NULL)
     {
@@ -1149,7 +1208,7 @@ hd_desktop_load_containers (HDDesktop *desktop)
                                         "height", height,
                                         "orientation", orientation,
                                         "stretch", TRUE,
-                                         NULL);
+                                         NULL);        
       }
       else if (!g_ascii_strcasecmp (type, HD_CONTAINER_TYPE_PANEL_EXPANDABLE))
       {
@@ -1168,6 +1227,24 @@ hd_desktop_load_containers (HDDesktop *desktop)
                                          NULL);
       }
 
+      if (gtk_widget_get_direction (info->container) == GTK_TEXT_DIR_RTL)
+      {
+        HildonDesktopPanelWindowOrientation new_orientation;
+        gint new_x;
+      
+        /* Mirror the values in RTL mode */
+        new_x = gdk_screen_get_width (gdk_screen_get_default ()) - x - width;
+
+        new_orientation = (orientation == HILDON_DESKTOP_PANEL_WINDOW_ORIENTATION_LEFT ? 
+                           HILDON_DESKTOP_PANEL_WINDOW_ORIENTATION_RIGHT :
+                           HILDON_DESKTOP_PANEL_WINDOW_ORIENTATION_LEFT);
+
+        g_object_set (info->container, 
+                      "x", new_x, 
+                      "orientation", new_orientation, 
+                      NULL);
+      }
+
       g_free (orientation_str);
     }
     else 
@@ -1220,6 +1297,8 @@ hd_desktop_load_containers (HDDesktop *desktop)
 
     g_free (type);
     g_free (plugin_dir);
+    g_free (container_config);
+    g_free (policy_module);
     g_list_foreach (plugin_list, (GFunc) g_free , NULL);
     g_list_free (plugin_list);
   }
