@@ -41,6 +41,8 @@
 #include <glib/gkeyfile.h>
 #include <glib/gstdio.h>
 
+#undef DEBUG
+
 /* Key used in the home-background.conf */
 #define HD_HOME_BACKGROUND_KEY_GROUP            "Hildon Home"
 #define HD_HOME_BACKGROUND_KEY_URI              "BackgroundImage"
@@ -364,6 +366,9 @@ hd_home_background_requires_caching (HDHomeBackground *background)
 
   g_return_val_if_fail (filename, FALSE);
 
+  if (filename[0] == '\0')
+    return FALSE;
+
   mmc_mount_point = g_getenv (HD_ENV_MMC_MOUNTPOINT);
   mmc_mount_point_uri = g_strdup_printf ("file://%s", mmc_mount_point);
 
@@ -377,10 +382,12 @@ hd_home_background_requires_caching (HDHomeBackground *background)
   on_external_fs = filename[0] != G_DIR_SEPARATOR &&
       (!g_str_has_prefix (filename, "file://"));
 
+#ifdef DEBUG
   g_debug ("filename %s requires caching: %i",
            filename,
            (on_internal_mmc || on_external_fs));
-
+#endif
+  
   return (on_internal_mmc || on_external_fs);
 
 
@@ -560,9 +567,10 @@ hd_home_background_apply_async (HildonDesktopBackground        *background,
                 "mode", &mode,
                 NULL);
 
+#ifdef DEBUG
   g_debug ("Applying background %s aynchronously, cache: %s",
            filename, cache);
-
+#endif
 
   if (cache && *cache)
   {
