@@ -34,7 +34,6 @@
 #include <libosso.h>
 #endif
 
-
 #include <libhildonwm/hd-wm.h>
 
 #include <libhildondesktop/hildon-home-area.h>
@@ -46,6 +45,10 @@
 
 #include <dbus/dbus-glib.h>
 
+#include <stdio.h>
+#include <sys/types.h>
+#include <signal.h>
+
 #include <X11/Xatom.h>
 #include <X11/extensions/Xrender.h>
 
@@ -74,6 +77,8 @@
 #define HD_HOME_BACKGROUND_DEFAULT          HD_DESKTOP_BACKGROUNDS_PATH\
                                             "/default.desktop"
 
+#undef DEBUG
+
 #define BG_DESKTOP_GROUP           "Desktop Entry"
 #define BG_DESKTOP_IMAGE_FILENAME  "File"
 
@@ -98,17 +103,9 @@ struct _HDHomeWindowPrivate
   osso_context_t               *osso_context;
 #endif
 
-  gboolean                     layout_mode_sucks;
-
   GtkWidget                    *normal_menu;
-  GtkWidget                    *layout_menu;
-
   GtkWidget                    *settings_item;
   GtkWidget                    *settings_menu;
-  GtkWidget                    *layout_mode_item;
-
-  GtkWidget                    *layout_mode_banner;
-  gint                          layout_mode_banner_to;
 
   HildonDesktopBackground      *background;
   HildonDesktopBackground      *previous_background;
@@ -175,44 +172,14 @@ hd_home_window_set_background_activate (HDHomeWindow *window);
 #ifndef HAVE_LIBHILDON
 static void
 hd_home_window_settings_insensitive_press_cb (HDHomeWindow *window);
-
-static void
-hd_home_window_layout_insensitive_press_cb (HDHomeWindow *window);
 #endif
 
-static void
-hd_home_window_layout_mode_activate (HDHomeWindow *window);
-
 static GtkWidget *
 hd_home_window_build_main_menu (HDHomeWindow *window);
 
-static GtkWidget *
-hd_home_window_build_layout_menu (HDHomeWindow *window);
-
 static void
 hd_home_window_ensure_menu_status (HDHomeWindow *window);
 
-static void
-hd_home_window_cancel_layout (HDHomeWindow *window);
-
-static void
-hd_home_window_accept_layout (HDHomeWindow *window);
-
-static void
-hd_home_window_layout_mode_accept (HDHomeWindow *window);
-
-static void
-hd_home_window_layout_mode_cancel (HDHomeWindow *window);
-
-static void
-hd_home_window_snap_toggled (HDHomeWindow *window, GtkCheckMenuItem *item);
-
-static void
-hd_home_window_show_layout_mode_banner (HDHomeWindow *window);
-
-static gboolean
-hd_home_window_destroy_banner (HDHomeWindow *window);
-
 static void
 hd_home_window_set_background (HDHomeWindow *window,
                                HildonDesktopBackground *background);
@@ -266,24 +233,12 @@ hd_home_window_area_remove (HDHomeWindow *window);
 static void
 hd_home_window_area_changed (HDHomeWindow *window);
 
-static void
-hd_home_window_layout_mode_started (HDHomeWindow *window);
-
-static void
-hd_home_window_layout_mode_ended (HDHomeWindow *window);
-
 static void
 hd_home_window_select_applets (HDHomeWindow *window);
 
 static void
 hd_home_window_load_area_layout (HDHomeWindow *window);
 
-static void
-hd_home_window_button1_clicked (HildonHomeWindow *window);
-
-static void
-hd_home_window_button2_clicked (HildonHomeWindow *window);
-
 static void
 hd_home_window_cancel_note_hide (HDHomeWindow *window);
 
@@ -319,11 +274,6 @@ hd_home_window_class_init (HDHomeWindowClass *window_class)
   widget_class->unrealize = hd_home_window_unrealize;
   widget_class->expose_event = hd_home_window_expose;
 
-  hhwindow_class->button1_clicked = hd_home_window_button1_clicked;
-  hhwindow_class->button2_clicked = hd_home_window_button2_clicked;
-
-  window_class->layout_mode_accept = hd_home_window_layout_mode_accept;
-  window_class->layout_mode_cancel = hd_home_window_layout_mode_cancel;
   window_class->background         = hd_home_window_background;
   window_class->screen_off         = hd_home_window_screen_off;
 
@@ -344,15 +294,6 @@ hd_home_window_class_init (HDHomeWindowClass *window_class)
                                    PROP_BACKGROUND,
                                    pspec);
 
-  pspec = g_param_spec_boolean ("layout-mode-sucks",
-                                "Layout mode sucks",
-                                "Whether or not the layout mode sucks",
-                                TRUE,
-                               (G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-  g_object_class_install_property (object_class,
-                                   PROP_LAYOUT_MODE_SUCKS,
-                                   pspec);
-
   g_signal_new ("background",
                 G_OBJECT_CLASS_TYPE (object_class),
                 G_SIGNAL_RUN_FIRST,
@@ -386,26 +327,6 @@ hd_home_window_class_init (HDHomeWindowClass *window_class)
                 1,
                 G_TYPE_BOOLEAN);
 
-  g_signal_new ("layout-mode-accept",
-                G_OBJECT_CLASS_TYPE (object_class),
-                G_SIGNAL_RUN_FIRST,
-                G_STRUCT_OFFSET (HDHomeWindowClass, layout_mode_accept),
-                NULL,
-                NULL,
-                g_cclosure_marshal_VOID__VOID,
-                G_TYPE_NONE,
-                0);
-
-  g_signal_new ("layout-mode-cancel",
-                G_OBJECT_CLASS_TYPE (object_class),
-                G_SIGNAL_RUN_FIRST,
-                G_STRUCT_OFFSET (HDHomeWindowClass, layout_mode_cancel),
-                NULL,
-                NULL,
-                g_cclosure_marshal_VOID__VOID,
-                G_TYPE_NONE,
-                0);
-
   g_signal_new ("io-error",
                 G_OBJECT_CLASS_TYPE (object_class),
                 G_SIGNAL_RUN_FIRST,
@@ -455,8 +376,6 @@ hd_home_window_constructor (GType                   gtype,
   priv->normal_menu = hd_home_window_build_main_menu (window);
   hildon_home_window_set_menu (hhwindow, GTK_MENU (priv->normal_menu));
 
-  priv->layout_menu = hd_home_window_build_layout_menu (window);
-
   g_object_set (window,
                 "menu", priv->normal_menu,
                 "title", HH_MENU_TITLE,
@@ -476,25 +395,10 @@ hd_home_window_constructor (GType                   gtype,
                             G_CALLBACK (hd_home_window_area_remove),
                             window);
 
-  g_signal_connect_swapped (area, "layout-mode-start",
-                            G_CALLBACK (hd_home_window_show_layout_mode_banner),
+  g_signal_connect_swapped (area, "layout-changed",
+                            G_CALLBACK (hd_home_window_area_changed),
                             window);
 
-  g_signal_connect_swapped (area, "layout-mode-started",
-                            G_CALLBACK (hd_home_window_layout_mode_started),
-                            window);
-
-  g_signal_connect_swapped (area, "layout-mode-ended",
-                            G_CALLBACK (hd_home_window_layout_mode_ended),
-                            window);
-
-  if (priv->layout_mode_sucks)
-    {
-      g_signal_connect_swapped (area, "layout-changed",
-                                G_CALLBACK (hd_home_window_area_changed),
-                                window);
-    }
-
   background = g_object_new (HD_TYPE_HOME_BACKGROUND, NULL);
 
   conffile = g_build_path (G_DIR_SEPARATOR_S,
@@ -507,7 +411,6 @@ hd_home_window_constructor (GType                   gtype,
                                   conffile,
                                   &error);
 
-  g_free (conffile);
 
   if (error)
     {
@@ -517,9 +420,11 @@ hd_home_window_constructor (GType                   gtype,
 
       g_clear_error (&error);
 
+#ifdef DEBUG
       g_debug ("Got an error when loading background configuration, "
                "reverting to default background.");
-
+#endif
+      
       /* Revert to the default background file */
       keyfile = g_key_file_new ();
       g_key_file_load_from_file (keyfile,
@@ -540,6 +445,10 @@ hd_home_window_constructor (GType                   gtype,
                                  "color", &default_color,
                                  NULL);
 
+      hildon_desktop_background_save (background,
+                                      conffile,
+                                      NULL);
+
 error:
       if (error)
         {
@@ -556,6 +465,8 @@ error:
 
     }
 
+  g_free (conffile);
+
 
   if (background)
     hd_home_window_set_background (window, background);
@@ -576,7 +487,6 @@ hd_home_window_set_property (GObject      *gobject,
                              GParamSpec   *pspec)
 {
   HDHomeWindow *window = HD_HOME_WINDOW (gobject);
-  HDHomeWindowPrivate *priv = HD_HOME_WINDOW_GET_PRIVATE (gobject);
 
   switch (prop_id)
   {
@@ -594,10 +504,6 @@ hd_home_window_set_property (GObject      *gobject,
                                      g_value_get_object (value));
       break;
 
-    case PROP_LAYOUT_MODE_SUCKS:
-      priv->layout_mode_sucks = g_value_get_boolean (value);
-      break;
-
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
@@ -620,10 +526,6 @@ hd_home_window_get_property (GObject    *gobject,
       break;
 #endif
 
-    case PROP_LAYOUT_MODE_SUCKS:
-      g_value_set_boolean (value, priv->layout_mode_sucks);
-      break;
-
     case PROP_BACKGROUND:
       g_value_set_object (value, priv->background);
       break;
@@ -634,6 +536,31 @@ hd_home_window_get_property (GObject    *gobject,
   }
 }
 
+static guint
+pid_from_exec_name (char *exec_name)
+{
+  FILE *pfile = NULL;
+  gchar *cmdline = NULL;
+  guint the_pid = 0;
+
+  if ((cmdline = g_strdup_printf ("pidof %s", exec_name)) != NULL) 
+  {
+    if ((pfile = popen (cmdline, "r")) != NULL) 
+    {
+      fscanf (pfile, "%d", &the_pid);
+      pclose (pfile);
+    }
+    else
+    {
+      return 0;
+    }
+
+    g_free (cmdline);
+  }
+
+  return the_pid;
+}
+
 static void
 background_apply_callback (HildonDesktopBackground     *background,
                            Picture                      picture,
@@ -641,8 +568,12 @@ background_apply_callback (HildonDesktopBackground     *background,
                            HDHomeWindow                *window)
 {
   HDHomeWindowPrivate  *priv;
-  g_debug ("Background applied!");
+  guint greeting_pid;
 
+#ifdef DEBUG
+  g_debug ("Background applied!");
+#endif
+  
   priv = HD_HOME_WINDOW_GET_PRIVATE (window);
 
   if (priv->background_loading_timeout > 0)
@@ -673,8 +604,10 @@ background_apply_callback (HildonDesktopBackground     *background,
                                        "org.freedesktop.DBus.GLib.UnmappedError.BackgroundManagerErrorQuark.Code1")))
         text = HH_NO_CONNECTION_TEXT;
       else if ((dbus_g_error_has_name (error,
-                                       "org.freedesktop.DBus.GLib.UnmappedError.BackgroundManagerErrorQuark.Code4")))
-        text = HH_MMC_OPEN_TEXT;
+                                       "org.freedesktop.DBus.GLib.UnmappedError.BackgroundManagerErrorQuark.Code3")))
+        hildon_banner_show_information (GTK_WIDGET (window),
+                                        NULL,
+                                        HH_NO_OPEN);
       else if ((dbus_g_error_has_name (error,
                                        "org.freedesktop.DBus.GLib.UnmappedError.BackgroundManagerErrorQuark.Code6")))
         text = HH_FLASH_FULL_TEXT;
@@ -684,6 +617,12 @@ background_apply_callback (HildonDesktopBackground     *background,
       else if ((dbus_g_error_has_name (error,
                                        "org.freedesktop.DBus.GLib.UnmappedError.GdkPixbufErrorQuark.Code1")))
         text = HH_LOW_MEMORY_TEXT;
+      else if ((dbus_g_error_has_name (error,
+                                       "org.freedesktop.DBus.GLib.UnmappedError.GdkPixbufErrorQuark.Code3")))
+        text = HH_FILE_CORRUPTED_TEXT;
+      else if ((dbus_g_error_has_name (error,
+                                       "org.freedesktop.DBus.GLib.UnmappedError.GdkPixbufErrorQuark.Code5")))
+        text = HH_FILE_CORRUPTED_TEXT;
 
       if (text)
         hd_home_window_show_information_note (window, text);
@@ -695,14 +634,16 @@ background_apply_callback (HildonDesktopBackground     *background,
     XRenderFreePicture (GDK_DISPLAY (), priv->background_picture);
 
   priv->background_picture = picture;
-  g_debug ("Received picture %i", (gint)priv->background_picture);
 
+#ifdef DEBUG
+  g_debug ("Received picture %i", (gint)priv->background_picture);
+#endif
+  
   if (GTK_WIDGET_VISIBLE (GTK_WIDGET (window)))
     gdk_window_invalidate_rect (GTK_WIDGET (window)->window,
                                 (GdkRectangle*)(&GTK_WIDGET (window)->allocation),
                                 TRUE);
 
-
   if (background != priv->background)
     {
       if (priv->background)
@@ -710,6 +651,11 @@ background_apply_callback (HildonDesktopBackground     *background,
       priv->background = g_object_ref (background);
     }
 
+  /* If there's a process osso_startup_greeting, kill it with SIGUSR1. */
+  greeting_pid = pid_from_exec_name ("osso_startup_greeting"); 
+
+  if (greeting_pid > 0)
+    kill (greeting_pid, SIGUSR1);
 }
 
 static void
@@ -946,11 +892,6 @@ hd_home_window_key_press_event (GtkWidget *widget,
     {
       case HILDON_HARDKEY_MENU:
           hildon_home_window_toggle_menu (HILDON_HOME_WINDOW (widget));
-          break;
-      case HILDON_HARDKEY_ESC:
-          if (!priv->layout_mode_sucks)
-            hd_home_window_cancel_layout (HD_HOME_WINDOW (widget));
-
           break;
       default:
             return GTK_WIDGET_CLASS (
@@ -1035,28 +976,7 @@ hd_home_window_build_main_menu (HDHomeWindow *window)
 #endif
 #endif
 
-  if (!priv->layout_mode_sucks)
-    {
-      /* layout mode */
-      menu_item = gtk_menu_item_new_with_label (HH_MENU_EDIT_LAYOUT);
-      g_signal_connect_swapped (menu_item, "activate",
-                                G_CALLBACK (hd_home_window_layout_mode_activate),
-                                window);
-#ifdef HAVE_LIBHILDON
-      hildon_helper_set_insensitive_message (menu_item, HH_LAYOUT_UNAVAIL_BANNER);
-#else
-      g_signal_connect_swapped (menu_item, "insensitive-press",
-                                G_CALLBACK (hd_home_window_layout_insensitive_press_cb),
-                                window);
-#endif
-      gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
-      gtk_widget_set_sensitive (menu_item, FALSE);
-      gtk_widget_show (menu_item);
-      priv->layout_mode_item = menu_item;
-    }
-
 #if 0
-  else
     {
       GtkWidget *area;
 
@@ -1088,62 +1008,6 @@ hd_home_window_build_main_menu (HDHomeWindow *window)
 
 }
 
-static GtkWidget *
-hd_home_window_build_layout_menu (HDHomeWindow *window)
-{
-  GtkWidget    *menu;
-  GtkWidget    *mi;
-  GtkWidget    *area;
-
-  menu = gtk_menu_new ();
-
-  mi = gtk_menu_item_new_with_label (HH_MENU_LAYOUT_SELECT);
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
-  g_signal_connect_swapped (mi, "activate",
-                            G_CALLBACK (hd_home_window_select_applets),
-                            window);
-  gtk_widget_show (mi);
-
-  mi = gtk_menu_item_new_with_label (HH_MENU_LAYOUT_ACCEPT);
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
-  g_signal_connect_swapped (mi, "activate",
-                            G_CALLBACK (hd_home_window_accept_layout),
-                            window);
-  gtk_widget_show (mi);
-
-  mi = gtk_menu_item_new_with_label (HH_MENU_LAYOUT_CANCEL);
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
-  g_signal_connect_swapped (mi, "activate",
-                            G_CALLBACK (hd_home_window_cancel_layout),
-                            window);
-  gtk_widget_show (mi);
-
-  mi = gtk_check_menu_item_new_with_label (HH_MENU_LAYOUT_SNAP_TO_GRID);
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
-  g_signal_connect_swapped (mi, "toggled",
-                            G_CALLBACK (hd_home_window_snap_toggled),
-                            window);
-  area = GTK_BIN (window)->child;
-  if (HILDON_IS_HOME_AREA (area))
-    {
-      gboolean snap_to_grid;
-      g_object_get (G_OBJECT (area), "snap-to-grid", &snap_to_grid, NULL);
-      gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (mi), snap_to_grid);
-    }
-  gtk_widget_show (mi);
-
-#ifdef HAVE_LIBHILDONHELP
-  mi = gtk_menu_item_new_with_label (HH_MENU_LAYOUT_HELP);
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
-  g_signal_connect_swapped (mi, "activate",
-                            G_CALLBACK (hd_home_window_help_activate),
-                            window);
-  gtk_widget_show (mi);
-#endif
-
-  return menu;
-}
-
 #ifdef HAVE_LIBOSSO
 static void
 hd_home_window_applet_activate (HDHomeWindow *window,
@@ -1157,7 +1021,10 @@ hd_home_window_applet_activate (HDHomeWindow *window,
   if (!priv->osso_context)
     return;
 
+#ifdef DEBUG
   g_debug ("Launching controlpanel applet %s", applet_path);
+#endif
+  
   res = osso_cp_plugin_execute (priv->osso_context,
                                 applet_path,
                                 NULL,
@@ -1203,14 +1070,9 @@ hd_home_window_help_activate (HDHomeWindow *window)
   if (!priv->osso_context)
     return;
 
-  if (hildon_home_area_get_layout_mode (area))
-    res = hildon_help_show (priv->osso_context,
-                            HH_HELP_LAYOUT_MODE,
-                            HILDON_HELP_SHOW_DIALOG);
-  else
-    res = hildon_help_show (priv->osso_context,
-                            HH_HELP,
-                            0);
+  res = hildon_help_show (priv->osso_context,
+                          HH_HELP,
+                          0);
 
   switch (res)
   {
@@ -1264,13 +1126,6 @@ hd_home_window_settings_insensitive_press_cb (HDHomeWindow *window)
 
 }
 
-static void
-hd_home_window_layout_insensitive_press_cb (HDHomeWindow *window)
-{
-  hd_home_window_show_information_banner (window,
-                                          HH_LAYOUT_UNAVAIL_BANNER);
-
-}
 #endif
 
 static void
@@ -1278,8 +1133,10 @@ hd_home_window_save_area_layout (HDHomeWindow *window)
 {
   GtkWidget    *area = GTK_BIN (window)->child;
 
+#ifdef DEBUG
   g_debug ("Saving area layout");
-
+#endif
+  
   if (HILDON_IS_HOME_AREA (area))
     {
       GError *error = NULL;
@@ -1319,8 +1176,26 @@ hd_home_window_load_area_layout (HDHomeWindow *window)
       if (g_file_test (user_filename, G_FILE_TEST_EXISTS))
         filename = user_filename;
       else
+      {
+        /* If there is no local file, we use the system-wide one
+         * and copy it locally */
+        gchar *content;
+        gsize length;
+
         filename = HH_AREA_GLOBAL_CONFIGURATION_FILE;
 
+        if (g_file_get_contents (filename,
+                                 &content,
+                                 &length,
+                                 NULL))
+        {
+          g_file_set_contents (user_filename,
+                               content,
+                               length,
+                               NULL);
+        }
+      }
+
       hildon_home_area_load_configuration (HILDON_HOME_AREA (area),
                                            filename,
                                            &error);
@@ -1336,33 +1211,6 @@ hd_home_window_load_area_layout (HDHomeWindow *window)
 
 }
 
-static void
-hd_home_window_button1_clicked (HildonHomeWindow *window)
-{
-  hd_home_window_accept_layout (HD_HOME_WINDOW (window));
-}
-
-static void
-hd_home_window_button2_clicked (HildonHomeWindow *window)
-{
-  hd_home_window_cancel_layout (HD_HOME_WINDOW (window));
-}
-
-
-static void
-hd_home_window_snap_toggled (HDHomeWindow *window, GtkCheckMenuItem *item)
-{
-  GtkWidget *area;
-
-  area = GTK_BIN (window)->child;
-
-  if (HILDON_IS_HOME_AREA (area))
-    g_object_set (area,
-                  "snap-to-grid", gtk_check_menu_item_get_active (item),
-                  NULL);
-
-}
-
 static void
 hd_home_window_area_add (HDHomeWindow *window)
 {
@@ -1372,14 +1220,6 @@ hd_home_window_area_add (HDHomeWindow *window)
   priv = HD_HOME_WINDOW_GET_PRIVATE (window);
   area = GTK_BIN (window)->child;
 
-  if (!priv->layout_mode_sucks)
-    {
-      if (priv->selecting_applets &&
-          !hildon_home_area_get_layout_mode (HILDON_HOME_AREA (area)))
-        hildon_home_area_set_layout_mode (HILDON_HOME_AREA (area), TRUE);
-
-    }
-
   hd_home_window_ensure_menu_status (window);
 }
 
@@ -1392,36 +1232,9 @@ hd_home_window_area_remove (HDHomeWindow *window)
   priv = HD_HOME_WINDOW_GET_PRIVATE (window);
   area = GTK_BIN (window)->child;
 
-  if (!hildon_home_area_get_layout_mode (HILDON_HOME_AREA (area)))
-    hd_home_window_save_area_layout (window);
+  hd_home_window_save_area_layout (window);
 
   hd_home_window_ensure_menu_status (window);
-
-}
-
-static void
-hd_home_window_layout_mode_started (HDHomeWindow *window)
-{
-  HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (window);
-
-  g_object_set (window,
-                "title", HH_MENU_LAYOUT_TITLE,
-                "menu",  priv->layout_menu,
-                NULL);
-}
-
-static void
-hd_home_window_layout_mode_ended (HDHomeWindow *window)
-{
-  HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (window);
-
-  hd_home_window_destroy_banner (window);
-
-  g_object_set (window,
-                "title", HH_MENU_TITLE,
-                "menu",  priv->normal_menu,
-                NULL);
-
 }
 
 static gboolean
@@ -1480,11 +1293,6 @@ hd_home_window_ensure_menu_status (HDHomeWindow *window)
   items = gtk_container_get_children (GTK_CONTAINER (area));
   if (!items)
     {
-      /* if no applets are set we disable the layout_mode item
-       * and the settings item.
-       */
-      if (!priv->layout_mode_sucks)
-        gtk_widget_set_sensitive (priv->layout_mode_item, FALSE);
       gtk_widget_set_sensitive (priv->settings_item, FALSE);
       return;
     }
@@ -1499,6 +1307,7 @@ hd_home_window_ensure_menu_status (HDHomeWindow *window)
       applet = HILDON_DESKTOP_HOME_ITEM (l->data);
 
       item = hildon_desktop_home_item_get_settings_menu_item (applet);
+
       if (item && GTK_IS_MENU_ITEM (item))
         {
           gtk_menu_append (GTK_MENU (priv->settings_menu), item);
@@ -1508,8 +1317,6 @@ hd_home_window_ensure_menu_status (HDHomeWindow *window)
         }
     }
 
-  if (!priv->layout_mode_sucks)
-    gtk_widget_set_sensitive (priv->layout_mode_item, TRUE);
   gtk_widget_set_sensitive (priv->settings_item, settings_item_active);
   g_list_free (items);
 }
@@ -1535,11 +1342,10 @@ hd_home_window_osso_display_state_cb (osso_display_state_t      state,
   /* We don't do anything when the screen is dimmed, only on and off */
   if (state == OSSO_DISPLAY_DIMMED)
     return;
-
+  
   g_signal_emit_by_name (G_OBJECT (window),
                          "screen-off",
                          state == OSSO_DISPLAY_OFF);
-
 }
 
 static void
@@ -1572,118 +1378,6 @@ hd_home_window_set_osso_context (HDHomeWindow *window,
 }
 #endif
 
-static gboolean
-hd_home_window_destroy_banner (HDHomeWindow *window)
-{
-  HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (window);
-  GtkWidget            *area = GTK_BIN (window)->child;
-
-  if (priv->layout_mode_banner)
-    {
-      gtk_widget_destroy (priv->layout_mode_banner);
-      priv->layout_mode_banner = NULL;
-      g_signal_handlers_disconnect_by_func (G_OBJECT (area),
-                                            G_CALLBACK (hd_home_window_destroy_banner),
-                                            window);
-    }
-  priv->layout_mode_banner_to = 0;
-
-  return FALSE;
-}
-
-static void
-hd_home_window_show_layout_mode_banner (HDHomeWindow *window)
-{
-  HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (window);
-  GtkWidget            *area = GTK_BIN (window)->child;
-
-  if (!priv->layout_mode_banner)
-    {
-      priv->layout_mode_banner =
-          hildon_banner_show_animation (GTK_WIDGET (window),
-                                        NULL,
-                                        HH_LAYOUT_MODE_BANNER);
-      priv->layout_mode_banner_to =
-          g_timeout_add (LAYOUT_OPENING_BANNER_TIMEOUT,
-                         (GSourceFunc)hd_home_window_destroy_banner,
-                         window);
-
-      g_signal_connect_swapped (G_OBJECT (area),
-                                "applet-change-start",
-                                G_CALLBACK (hd_home_window_destroy_banner),
-                                window);
-
-
-    }
-
-}
-
-static void
-hd_home_window_layout_mode_accept (HDHomeWindow *window)
-{
-  HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (window);
-  GtkWidget            *area;
-
-  area = GTK_BIN (window)->child;
-  g_return_if_fail (HILDON_IS_HOME_AREA (area));
-
-  if (hildon_home_area_get_overlaps (HILDON_HOME_AREA (area)))
-    {
-      hd_home_window_show_information_note (HD_HOME_WINDOW (window),
-                                            HH_LAYOUT_OVERLAP_TEXT);
-      return;
-    }
-
-  if (!priv->layout_mode_sucks)
-    {
-      hildon_home_area_set_layout_mode (HILDON_HOME_AREA (area),
-                                        FALSE);
-    }
-
-  hd_home_window_save_area_layout (HD_HOME_WINDOW (window));
-  g_signal_emit_by_name (window, "save", NULL);
-
-}
-
-static void
-hd_home_window_layout_mode_cancel (HDHomeWindow *window)
-{
-  HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (window);
-  GtkWidget            *area;
-
-  area = GTK_BIN (window)->child;
-  g_return_if_fail (HILDON_IS_HOME_AREA (area));
-
-  if (hildon_home_area_get_layout_changed (HILDON_HOME_AREA (area)))
-    {
-      GtkWidget *note;
-      gint response;
-
-      note = hildon_note_new_confirmation_add_buttons
-          (NULL,
-           HH_LAYOUT_CANCEL_TEXT,
-           HH_LAYOUT_CANCEL_YES,
-           GTK_RESPONSE_ACCEPT,
-           HH_LAYOUT_CANCEL_NO,
-           GTK_RESPONSE_CANCEL,
-           NULL);
-
-      response = gtk_dialog_run (GTK_DIALOG (note));
-      gtk_widget_destroy (note);
-
-      if (response != GTK_RESPONSE_ACCEPT)
-        return;
-    }
-
-  hd_home_window_load_area_layout (HD_HOME_WINDOW (window));
-  g_signal_emit_by_name (window, "load", NULL);
-
-  if (!priv->layout_mode_sucks)
-    hildon_home_area_set_layout_mode (HILDON_HOME_AREA (area),
-                                      FALSE);
-
-}
-
 static void
 hd_home_window_background (HDHomeWindow    *window,
                            gboolean             is_background)
@@ -1694,24 +1388,15 @@ hd_home_window_background (HDHomeWindow    *window,
   area = GTK_BIN (window)->child;
   g_return_if_fail (HILDON_IS_HOME_AREA (area));
 
+#ifdef DEBUG
   g_debug ("Background state changed, is_background: %i", is_background);
-
+#endif
+  
   priv->is_background = is_background;
 
   if (!priv->screen_is_off)
     {
 
-      /* If we were in layout mode and went to background, we need
-       * to cancel it */
-      if (is_background &&
-          hildon_home_area_get_layout_mode (HILDON_HOME_AREA (area)))
-        {
-          hildon_home_area_set_layout_mode (HILDON_HOME_AREA (area),
-                                            FALSE);
-
-          hd_home_window_save_area_layout (window);
-        }
-
       gtk_container_foreach (GTK_CONTAINER (area),
                              (GtkCallback)hildon_desktop_home_item_set_is_background,
                              (gpointer)is_background);
@@ -1727,13 +1412,14 @@ hd_home_window_screen_off (HDHomeWindow         *window,
   GtkWidget            *area;
 
   area = GTK_BIN (window)->child;
-  g_return_if_fail (HILDON_IS_HOME_AREA (area));
 
   priv->screen_is_off = is_off;
 
+#ifdef DEBUG
   g_debug ("Screen state changed, is_off: %i", is_off);
-
-  if (!priv->is_background)
+#endif
+  
+  if (!priv->is_background && HILDON_IS_HOME_AREA (area))
     {
       gtk_container_foreach (GTK_CONTAINER (area),
                              (GtkCallback)hildon_desktop_home_item_set_is_background,
@@ -1782,26 +1468,6 @@ hd_home_window_select_applets (HDHomeWindow *window)
   hildon_home_area_set_batch_add (HILDON_HOME_AREA (area), FALSE);
   priv->selecting_applets = FALSE;
 
-  if (HILDON_IS_HOME_AREA (area))
-    {
-      /* If we are in layout mode, we only save when layout mode is
-       * accepted */
-      if (!hildon_home_area_get_layout_mode (HILDON_HOME_AREA (area)))
-        g_signal_emit_by_name (window, "save", NULL);
-    }
-
-}
-
-static void
-hd_home_window_cancel_layout (HDHomeWindow *window)
-{
-  g_signal_emit_by_name (window, "layout-mode-cancel");
-}
-
-static void
-hd_home_window_accept_layout (HDHomeWindow *window)
-{
-  g_signal_emit_by_name (window, "layout-mode-accept");
 }
 
 typedef struct
@@ -1899,18 +1565,6 @@ hd_home_window_set_background_reponse (HDHomeWindow *window,
   g_object_unref (background);
 }
 
-static void
-hd_home_window_layout_mode_activate (HDHomeWindow *window)
-{
-  GtkWidget    *area = GTK_BIN (window)->child;
-
-  g_return_if_fail (HILDON_IS_HOME_AREA (area));
-
-  hildon_home_area_set_layout_mode (HILDON_HOME_AREA (area),
-                                    TRUE);
-
-}
-
 static void
 hd_home_window_set_background_activate (HDHomeWindow *window)
 {
