@@ -142,7 +142,9 @@ hd_panel_window_dialog_queued (HildonDesktopPanelExpandable *container,
 		      	       gpointer object)
 {
   gtk_widget_set_name (widget, HD_PANEL_WINDOW_DIALOG_BUTTON_NAME);
-  gtk_widget_set_name (GTK_BIN (widget)->child, HD_PANEL_WINDOW_DIALOG_BUTTON_NAME);
+  if (GTK_IS_BIN (widget) && GTK_IS_WIDGET (GTK_BIN (widget)->child))
+    gtk_widget_set_name (GTK_BIN (widget)->child,
+                         HD_PANEL_WINDOW_DIALOG_BUTTON_NAME);
 }
 
 static void 
@@ -162,6 +164,8 @@ hd_panel_window_dialog_constructor (GType gtype,
                              GObjectConstructParam  *params)
 {
   GObject *object;
+  GtkContainer *panel;
+  GtkWidget *extension;
 #ifdef HD_PANEL_WINDOW_DIALOG_NEW_THEME
   HildonDesktopPanelWindowOrientation orientation;
 #endif
@@ -199,6 +203,13 @@ hd_panel_window_dialog_constructor (GType gtype,
   
   hd_panel_window_dialog_set_style (HD_PANEL_WINDOW_DIALOG (object), orientation);
 #endif
+
+  panel = HILDON_DESKTOP_WINDOW (object)->container;
+
+  extension = 
+    hildon_desktop_panel_expandable_get_extension (HILDON_DESKTOP_PANEL_EXPANDABLE (panel));
+
+  gtk_widget_set_name (extension, "HildonStatusBarExtension");
   
   return object;
 }
