@@ -74,6 +74,7 @@
 #define AS_BUTTON_HEIGHT           38
 #define AS_ROW_HEIGHT 		   30
 #define AS_ICON_SIZE               26
+#define AS_GROUP_ICON_SIZE         16
 #define AS_CLOSE_BUTTON_SIZE       16
 #define AS_CLOSE_BUTTON_THUMB_SIZE 40
 #define AS_TOOLTIP_WIDTH           360
@@ -325,7 +326,7 @@ hd_switcher_menu_item_constructor (GType                  type,
   gtk_widget_show (hbox);
 
   priv->icon = gtk_image_new ();
-  gtk_box_pack_start (GTK_BOX (hbox), priv->icon, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), priv->icon, FALSE, FALSE, 10);
   gtk_widget_show (priv->icon);
 
   vbox = gtk_vbox_new (FALSE, 0);
@@ -347,11 +348,12 @@ hd_switcher_menu_item_constructor (GType                  type,
 
   if (priv->info)
   { 
-    GdkPixbuf *app_pixbuf;
-      
-    priv->is_blinking = hd_wm_entry_info_is_urgent (priv->info);
-    app_pixbuf = hd_wm_entry_info_get_icon (priv->info);
+    GdkPixbuf *app_pixbuf = NULL;
     
+    priv->is_blinking = hd_wm_entry_info_is_urgent (priv->info);
+
+    app_pixbuf = hd_wm_entry_info_get_icon (priv->info);
+
     if (!app_pixbuf)
     {
       GError *error = NULL;
@@ -539,9 +541,6 @@ hd_switcher_menu_item_size_request (GtkWidget      *widget,
     requisition->width = MAX (requisition->width, child_width);
 
   requisition->height = MAX (requisition->height, child_height);
-
-  g_debug ("menu-item requisition: (%d, %d)",
-	  requisition->width, requisition->height);
 }
 
 static void
@@ -748,6 +747,12 @@ hd_switcher_menu_item_class_init (HDSwitcherMenuItemClass *klass)
                                                         0,
                                                         NULL);
   
+  klass->bkilled_emblem = gtk_icon_theme_load_icon (icon_theme,
+                                                    "qgn_indi_bkilled",
+                                                    AS_GROUP_ICON_SIZE,
+                                                    0,
+                                                    NULL);
+  
   g_object_class_install_property (gobject_class,
 		  		   MENU_PROP_SHOW_CLOSE,
 				   g_param_spec_boolean ("show-close",
@@ -821,6 +826,48 @@ hd_switcher_menu_item_class_init (HDSwitcherMenuItemClass *klass)
   g_type_class_add_private (klass, sizeof (HDSwitcherMenuItemPrivate));
 }
 
+static GdkPixbuf *
+hd_switcher_menu_item_compose_bkill_pixbuf (HDSwitcherMenuItem *menuitem,
+				            const GdkPixbuf    *src,
+				            HDWMEntryInfo	     *info)
+{
+  HDSwitcherMenuItemClass *menuitem_class;
+  GdkPixbuf *retval;
+  gint dest_width, dest_height;
+  gint off_x, off_y;
+
+  g_return_val_if_fail (HD_IS_SWITCHER_MENU_ITEM (menuitem), NULL);
+  g_return_val_if_fail (GDK_IS_PIXBUF (src), NULL);
+  g_return_val_if_fail (info != NULL, NULL);
+
+  menuitem_class = HD_SWITCHER_MENU_ITEM_GET_CLASS (menuitem);
+
+  if (!menuitem_class->bkilled_emblem)
+    return NULL;
+
+  /* Make a copy of the source pixbuf, and also make
+   * sure that it has an alpha channel
+   */
+  retval = gdk_pixbuf_add_alpha (src, FALSE, 255, 255, 255);
+
+  dest_width = gdk_pixbuf_get_width (retval);
+  dest_height = gdk_pixbuf_get_height (retval);
+
+  off_x = dest_width - gdk_pixbuf_get_width (menuitem_class->bkilled_emblem);
+  off_y = dest_height - gdk_pixbuf_get_height (menuitem_class->bkilled_emblem);
+
+  gdk_pixbuf_composite (menuitem_class->bkilled_emblem, 
+		        retval,
+		        0, 0,
+			dest_width, dest_height,
+			off_x, off_y,
+			1.0, 1.0,
+			GDK_INTERP_BILINEAR,
+			0xFF);
+
+  return retval;
+}
+
 static void
 hd_switcher_menu_item_init (HDSwitcherMenuItem *menuitem)
 {
@@ -903,7 +950,7 @@ hd_switcher_menu_item_set_entry_info (HDSwitcherMenuItem *menuitem,
   		                      HDWMEntryInfo        *info)
 {
   HDSwitcherMenuItemPrivate *priv;
-  GdkPixbuf *pixbuf;
+  GdkPixbuf *pixbuf = NULL;
   
   g_return_if_fail (HD_IS_SWITCHER_MENU_ITEM (menuitem));
   g_return_if_fail (info != NULL);
@@ -911,24 +958,23 @@ hd_switcher_menu_item_set_entry_info (HDSwitcherMenuItem *menuitem,
   priv = menuitem->priv;
 
   priv->info = info;
-  
+
   pixbuf = hd_wm_entry_info_get_icon (priv->info);
   
   if (!pixbuf)
   {
     GError *error = NULL;
 
-
     pixbuf = hd_wm_entry_info_get_app_icon (priv->info,
                                            AS_ICON_THUMB_SIZE,
                                            &error);
+
     if (error)
     {
       g_warning ("Could not load icon %s from theme: %s.",
                  hd_wm_entry_info_get_app_icon_name (priv->info),
                  error->message);
           
-          
       g_clear_error (&error);
       
       pixbuf = gtk_icon_theme_load_icon (priv->icon_theme,
@@ -946,16 +992,38 @@ hd_switcher_menu_item_set_entry_info (HDSwitcherMenuItem *menuitem,
       }
     }
   }
+
   
   if (pixbuf)
   {
-    gtk_image_set_from_pixbuf (GTK_IMAGE (priv->icon), pixbuf);
+    GdkPixbuf *app_pixbuf;
+	  
+    if (hd_wm_entry_info_is_hibernating (info))
+    {
+      app_pixbuf = hd_switcher_menu_item_compose_bkill_pixbuf (menuitem, 
+		                                               pixbuf,
+							       info);
+    }
+    else
+    {
+      app_pixbuf = gdk_pixbuf_copy (pixbuf);
+    }
+
+    if (app_pixbuf)
+    {
+      gtk_image_set_from_pixbuf (GTK_IMAGE (priv->icon), app_pixbuf);
+      g_object_unref (app_pixbuf);
+    }
+ 
     g_object_unref (pixbuf);
   }
   
   gtk_label_set_text (GTK_LABEL (priv->label),
 		      hd_wm_entry_info_peek_title (priv->info));
 
+  gtk_label_set_text (GTK_LABEL (priv->label2), 
+                      hd_wm_entry_info_get_window_name (priv->info));
+
   g_object_notify (G_OBJECT (menuitem), "entry-info");
 }
 
