@@ -79,24 +79,24 @@
 #define ANIM_FPS      2
 
 #define AS_MENU_BUTTON_NAME "hildon-navigator-small-button4"
+#define AS_MENU_BUTTON_HIGHLIGHTED_NAME "hildon-navigator-small-button4-highlight"
 
-#define AS_MENU_BUTTON_HIGHLIGHTED "qgn_some_theme_to_be_defined"
 
 /* Hardcoded pixel perfecting values */
 #define AS_BUTTON_BORDER_WIDTH  0
 #define AS_MENU_BORDER_WIDTH    20
+#define AS_MENU_PANE_WIDTH      340
 #define AS_TIP_BORDER_WIDTH 	20
 #define AS_BUTTON_HEIGHT        38
-#define AS_MENU_BUTTON_HEIGHT   116
+#define AS_MENU_BUTTON_HEIGHT   60
 #define AS_ROW_HEIGHT 		30
-#define AS_ICON_SIZE            26
+#define AS_ICON_SIZE            40
 #define AS_TOOLTIP_WIDTH        360
 #define AS_MENU_ITEM_WIDTH      360
 #define AS_INTERNAL_PADDING     10
 #define AS_SEPARATOR_HEIGHT     10
 
 #define SWITCHER_HIGHLIGHTING_FREQ 1000
-#define SWITCHER_HIGHLIGHTED_NAME ""
 
 #define SWITCHER_BUTTON_SIZE 32
 #define SWITCHER_BUTTON_UP "qgn_list_tasknavigator_appswitcher"
@@ -162,6 +162,8 @@ struct _HDSwitcherMenuPrivate
 
   GtkTreeIter		   *last_iter_added;
 
+  guint                     blinking_notification;
+  
   HDWMEntryInfo		   *last_urgent_info;
 
   GtkWidget		   *window_dialog;
@@ -278,14 +280,17 @@ hd_switcher_menu_resize_menu (HildonDesktopPopupMenu *menu, HDSwitcherMenu *swit
 }	
 
 static GtkWidget *
-hd_switcher_menu_create_separator ()
+hd_switcher_menu_create_separator (gboolean is_application_menu)
 {
   GtkWidget *separator;
 
   separator = gtk_separator_menu_item_new ();
 
-  gtk_widget_set_size_request (separator, -1, 1);
-
+  if (is_application_menu)
+    gtk_widget_set_name (separator, HD_SWITCHER_MENU_APP_MENU_ITEM_NAME);
+  else
+    gtk_widget_set_name (separator, HD_SWITCHER_MENU_NOT_MENU_ITEM_NAME);
+	  
   return separator;
 }
 
@@ -332,11 +337,30 @@ hd_switcher_menu_popup_window_keypress_cb (GtkWidget      *widget,
 			                   GdkEventKey    *event,
 			                   HDSwitcherMenu *switcher)
 {
+  gboolean activate_menu = FALSE, cancel_menu = FALSE;
+  
   HildonDesktopPopupWindow *window =
     HILDON_DESKTOP_POPUP_WINDOW (widget);
 
-  if (event->keyval == GDK_Left ||
-      event->keyval == GDK_KP_Left ||
+  if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
+  {
+    if (event->keyval == GDK_Left ||
+        event->keyval == GDK_KP_Left)
+      activate_menu = TRUE;
+    else if (event->keyval == GDK_Right || 
+             event->keyval == GDK_KP_Right)
+      cancel_menu = TRUE;
+  } 
+  else 
+  {
+    if (event->keyval == GDK_Right)
+      activate_menu = TRUE;
+    else if (event->keyval == GDK_Left || 
+             event->keyval == GDK_KP_Left)
+      cancel_menu = TRUE;
+  }  
+
+  if (cancel_menu ||
       event->keyval == GDK_Escape)
   {
     hildon_desktop_popup_window_popdown (window);
@@ -364,8 +388,7 @@ hd_switcher_menu_popup_window_keypress_cb (GtkWidget      *widget,
     return TRUE;
   }
   else
-  if (event->keyval == GDK_Right ||
-      event->keyval == GDK_KP_Right)
+  if (activate_menu)
   {
     GList *notifications =
      hildon_desktop_popup_menu_get_children 
@@ -398,8 +421,22 @@ hd_switcher_menu_popup_window_pane_keypress_cb (GtkWidget      *widget,
 	                                        GdkEventKey    *event,
         	                                HDSwitcherMenu *switcher)
 {
-  if (event->keyval == GDK_Left ||
-      event->keyval == GDK_KP_Left)
+  gboolean cancel_menu = FALSE;
+  
+  if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
+  {
+    if (event->keyval == GDK_Right || 
+        event->keyval == GDK_KP_Right)
+      cancel_menu = TRUE;
+  } 
+  else 
+  {
+    if (event->keyval == GDK_Left || 
+        event->keyval == GDK_KP_Left)
+      cancel_menu = TRUE;
+  }  
+
+  if (cancel_menu)
   {
     hildon_desktop_popup_window_jump_to_pane
       (switcher->priv->popup_window, -1);
@@ -421,16 +458,34 @@ hd_switcher_menu_switcher_keypress_cb (GtkWidget      *widget,
 				       GdkEventKey    *event,
 				       HDSwitcherMenu *switcher)
 {
-  if (event->keyval == GDK_Right ||
-      event->keyval == GDK_KP_Right)
+  gboolean activate_menu = FALSE, cancel_menu = FALSE;
+  
+  if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
+  {
+    if (event->keyval == GDK_Left ||
+        event->keyval == GDK_KP_Left)
+      activate_menu = TRUE;
+    else if (event->keyval == GDK_Right || 
+             event->keyval == GDK_KP_Right)
+      cancel_menu = TRUE;
+  } 
+  else 
+  {
+    if (event->keyval == GDK_Right)
+      activate_menu = TRUE;
+    else if (event->keyval == GDK_Left || 
+             event->keyval == GDK_KP_Left)
+      cancel_menu = TRUE;
+  }  
+
+  if (activate_menu)
   {
     gtk_toggle_button_set_active
       (GTK_TOGGLE_BUTTON (switcher->priv->toggle_button), TRUE);
 
     return TRUE;
   }	  
-  else if (event->keyval == GDK_Left ||
-           event->keyval == GDK_KP_Left)
+  else if (cancel_menu)
   {
     gtk_widget_grab_focus (widget);
 
@@ -510,6 +565,10 @@ hd_switcher_menu_update_open (HildonDesktopPopupWindow *window,
   }
 #endif
 
+  hd_switcher_menu_changed_stack_cb (switcher->hdwm, 
+		                     HD_WM_ENTRY_INFO (hd_wm_get_active_window ()),
+				     switcher); 
+  
   hd_switcher_menu_scroll_to (window, switcher);
 }
 
@@ -606,7 +665,7 @@ hd_switcher_menu_add_notification_group (gpointer key,
     
     hildon_desktop_popup_menu_add_item
       (switcher->priv->menu_notifications,
-      GTK_MENU_ITEM (hd_switcher_menu_create_separator ()));
+      GTK_MENU_ITEM (hd_switcher_menu_create_separator (FALSE)));
 
     menu_item = hd_switcher_menu_item_new_from_notification_group
       (ngroup->notifications, icon, summary, ngroup->dbus_callback, TRUE);
@@ -771,7 +830,8 @@ hd_switcher_menu_constructor (GType gtype,
   switcher->priv->toggle_button = button;
 
   gtk_widget_set_name (button, AS_MENU_BUTTON_NAME);
-  gtk_widget_set_size_request (button, -1, AS_MENU_BUTTON_HEIGHT);
+  /*gtk_widget_set_size_request (button, -1, AS_MENU_BUTTON_HEIGHT);*/
+  gtk_widget_set_size_request (GTK_WIDGET (switcher), -1, AS_MENU_BUTTON_HEIGHT);
 
   pixbuf = hd_switcher_menu_get_icon_from_theme (switcher, AS_MENU_BUTTON_ICON, -1);
   switcher->priv->icon = gtk_image_new_from_pixbuf (pixbuf);
@@ -793,10 +853,11 @@ hd_switcher_menu_constructor (GType gtype,
   /* We don't attach the widget because if we do it, we cannot be on top of 
    * virtual keyboard. Anyway it should be transient to button
    */
-
+#if 0
   hildon_desktop_popup_window_attach_widget
     (switcher->priv->popup_window, NULL);	  
-  
+#endif
+
   switcher->priv->notifications_window = 
     hildon_desktop_popup_window_get_pane (switcher->priv->popup_window, 0);
 
@@ -805,17 +866,17 @@ hd_switcher_menu_constructor (GType gtype,
 		NULL);
   
   gtk_widget_set_size_request (GTK_WIDGET (switcher->priv->popup_window),
-			       340, 100);
+			       AS_MENU_PANE_WIDTH, 100);
   
   gtk_widget_set_size_request (GTK_WIDGET (switcher->priv->notifications_window),
-			       340, 100);  
+			       AS_MENU_PANE_WIDTH, 1);  
 
   switcher->priv->menu_applications =
     HILDON_DESKTOP_POPUP_MENU (g_object_new (HILDON_DESKTOP_TYPE_POPUP_MENU,
 		  		    	     "item-height", 72,
 		  		    	     "parent", switcher->priv->popup_window,
 				    	     NULL));	    
-
+  
   switcher->priv->menu_notifications =
     HILDON_DESKTOP_POPUP_MENU (g_object_new (HILDON_DESKTOP_TYPE_POPUP_MENU,
 		  			     "item-height", 72,
@@ -1251,8 +1312,8 @@ hd_switcher_menu_update_highlighting (HDSwitcherMenu *switcher, gboolean state)
 {
   if (state)
   {
-    gtk_widget_set_name (GTK_WIDGET (switcher), SWITCHER_HIGHLIGHTED_NAME);
-    gtk_widget_set_name (switcher->priv->toggle_button, SWITCHER_HIGHLIGHTED_NAME);    
+    gtk_widget_set_name (GTK_WIDGET (switcher), AS_MENU_BUTTON_HIGHLIGHTED_NAME);
+    gtk_widget_set_name (switcher->priv->toggle_button, AS_MENU_BUTTON_HIGHLIGHTED_NAME);
   }	  
   else
   {
@@ -1290,11 +1351,20 @@ hd_switcher_menu_check_content (HDSwitcherMenu *switcher)
         else
           hd_switcher_menu_attach_button (switcher);	     
       }	       
-	     
+
       hd_switcher_menu_update_highlighting (switcher, TRUE);
     }
     else
-    {	     
+    {
+      gtk_widget_hide (switcher->priv->notifications_window);
+
+      /* Avoid notification pane to be shown next time switcher
+       * menu opens. */
+      gtk_widget_set_size_request (switcher->priv->notifications_window,
+		                   AS_MENU_PANE_WIDTH, 1);
+
+      gtk_widget_queue_resize (switcher->priv->notifications_window);
+      
       hd_switcher_menu_update_highlighting (switcher, FALSE); 
     }
 
@@ -1333,7 +1403,7 @@ hd_switcher_menu_check_content (HDSwitcherMenu *switcher)
     hildon_desktop_popup_window_jump_to_pane (switcher->priv->popup_window, -1);
 
   g_list_free (children);
-}	
+}
 
 static void 
 hd_switcher_menu_position_func (HildonDesktopPopupWindow  *menu,
@@ -1440,16 +1510,12 @@ hd_switcher_menu_toggled_cb (GtkWidget *button, HDSwitcherMenu *switcher)
   {
     if (switcher->priv->is_open)	  
       hildon_desktop_popup_window_popdown (switcher->priv->popup_window);
-    switcher->priv->is_open = FALSE;
-    return;
-  }   
 
-  if (switcher->priv->is_open)
-  {	  
-    hildon_desktop_popup_menu_select_next_item (switcher->priv->menu_applications);
-    hd_wm_debug ("selecting next item");
-  }
-    
+    switcher->priv->is_open = FALSE;
+
+    return;
+  } 
+
   hildon_desktop_popup_window_popup 
    (switcher->priv->popup_window,
     hd_switcher_menu_position_func,
@@ -1476,12 +1542,13 @@ hd_switcher_menu_scroll_to (HildonDesktopPopupWindow *window,
 			    HDSwitcherMenu *switcher)
 {
   GtkAdjustment *adj;
-       
+
   if (!switcher->priv->menu_applications ||
       !switcher->priv->menu_notifications)
   {
     return;	   
-  }	   
+  }
+
   hildon_desktop_popup_menu_scroll_to_selected
    (switcher->priv->menu_applications);
 
@@ -1609,7 +1676,7 @@ hd_switcher_menu_create_notifications_menu (HDSwitcherMenu *switcher)
         if (first_item)
           hildon_desktop_popup_menu_add_item
             (switcher->priv->menu_notifications,
-            GTK_MENU_ITEM (hd_switcher_menu_create_separator ()));
+            GTK_MENU_ITEM (hd_switcher_menu_create_separator (FALSE)));
 
         menu_item =
           hd_switcher_menu_item_new_from_notification
@@ -1630,6 +1697,7 @@ hd_switcher_menu_create_notifications_menu (HDSwitcherMenu *switcher)
 
         if (!ack && !icon_is_set)
         {
+	  switcher->priv->blinking_notification = id;
           hd_switcher_menu_replace_blinking_icon (switcher, icon);
           icon_is_set = TRUE;
 	}
@@ -1679,12 +1747,14 @@ hd_switcher_menu_create_applications_menu (HDSwitcherMenu *switcher, HDWM *hdwm)
   for (l = apps; l != NULL; l = l->next)
   {
     GtkWidget *menu_item;
-    const GList * children = hd_wm_entry_info_get_children(l->data);
-    const GList * child;
+    GList *children, *child;
+
+    children = 
+      g_list_reverse (g_list_copy ((GList *) hd_wm_entry_info_get_children (l->data)));
 
     if (l == apps)
     {
-      separator = hd_switcher_menu_create_separator ();
+      separator = hd_switcher_menu_create_separator (TRUE);
 
       hildon_desktop_popup_menu_add_item
        (switcher->priv->menu_applications, GTK_MENU_ITEM (separator));
@@ -1717,11 +1787,13 @@ hd_switcher_menu_create_applications_menu (HDSwitcherMenu *switcher, HDWM *hdwm)
     /* Append the separator for this app*/
     if (l->next != NULL)
     {
-      separator = hd_switcher_menu_create_separator ();
+      separator = hd_switcher_menu_create_separator (TRUE);
 
       hildon_desktop_popup_menu_add_item
         (switcher->priv->menu_applications, GTK_MENU_ITEM (separator));
     }
+
+    g_list_free (children);
   }
 
   g_list_free (apps);
@@ -1836,6 +1908,26 @@ hd_switcher_get_default_icon_from_entry_info (HDSwitcherMenu *switcher, HDWMEntr
   return app_pixbuf;
 }	
 
+static void
+hd_switcher_menu_refresh_items (HDSwitcherMenu *switcher)
+{
+  GList *children = NULL, *l;
+
+  children =
+    hildon_desktop_popup_menu_get_children (switcher->priv->menu_applications);
+
+  for (l = children; l != NULL; l = g_list_next (l))
+  {
+    if (HD_IS_SWITCHER_MENU_ITEM (l->data))
+    {
+      HDWMEntryInfo *info =	    
+        hd_switcher_menu_item_get_entry_info (HD_SWITCHER_MENU_ITEM (l->data));
+
+      hd_switcher_menu_item_set_entry_info (HD_SWITCHER_MENU_ITEM (l->data), info);
+    }	    
+  }	  
+}
+	
 static void 
 hd_switcher_menu_changed_info_cb (HDWM *hdwm,
 				  HDWMEntryInfo *info,
@@ -1847,61 +1939,69 @@ hd_switcher_menu_changed_info_cb (HDWM *hdwm,
   gboolean make_it_blink = FALSE;
 
   if (!info)
-    return;	  
-
+  {
+    hd_switcher_menu_refresh_items (switcher);
+    return;
+  }
+    
   /* We have to guess whether it is in app switcher's slots or not*/
   
   if (HD_WM_IS_APPLICATION (info))
   {	  
-     apps = hd_wm_get_applications (switcher->hdwm);
+    apps = hd_wm_get_applications (switcher->hdwm);
 
-     for (l = apps; l != NULL; l = g_list_next (l))
-     {
-        HDWMEntryInfo *iter_info = (HDWMEntryInfo *) l->data;
+    for (l = apps; l != NULL; l = g_list_next (l))
+    {
+       HDWMEntryInfo *iter_info = (HDWMEntryInfo *) l->data;
 
-	if (!HD_WM_IS_APPLICATION (iter_info))
-          continue;
- 	
-        if (HD_WM_IS_APPLICATION (iter_info))		
-        {
-	  pos++;
+       if (HD_WM_IS_APPLICATION (iter_info))		
+       {
+         pos++;
 
-	  if (iter_info == info)
-            break;		  
-        }		
-     }
+         if (iter_info == info)
+           break;		  
+       }
+       else
+       {
+         continue;
+       }
+       
+    }
 
-     if (pos >= SWITCHER_N_SLOTS)
-       make_it_blink = TRUE;	    
+    if (pos >= SWITCHER_N_SLOTS)
+      make_it_blink = TRUE;	    
   }     
   else
   {
-     apps = hd_wm_get_applications (switcher->hdwm);
+    apps = hd_wm_get_applications (switcher->hdwm);
 
-     for (l = apps; l != NULL; l = g_list_next (l))
-     {
-       const GList *iter_children;	     
-       HDWMEntryInfo *iter_info = (HDWMEntryInfo *) l->data;
-       
-       if (HD_WM_IS_APPLICATION (iter_info))
-       {
-         pos++;	
-  	 
-         const GList *info_children = 
-  	   hd_wm_entry_info_get_children (iter_info);
-	 
-	 for (iter_children = info_children; 
- 	      iter_children != NULL; 
-	      iter_children = g_list_next (iter_children))
-         {
-           if (iter_children->data == info)
-	     break;	   
-         }		 
-       }	       
-     }	     
+    for (l = apps; l != NULL; l = g_list_next (l))
+    {
+      HDWMEntryInfo *iter_info = (HDWMEntryInfo *) l->data;
+      const GList *iter_children;	     
+      
+      if (HD_WM_IS_APPLICATION (iter_info))
+      {
+        pos++;	
+        
+        const GList *info_children = 
+          hd_wm_entry_info_get_children (iter_info);
+        
+        for (iter_children = info_children; 
+             iter_children != NULL; 
+             iter_children = g_list_next (iter_children))
+        {
+          if (iter_children->data == info)
+            break;   
+        }
 
-     if (pos >= SWITCHER_N_SLOTS)
-       make_it_blink = TRUE;	     
+        if (!iter_children)
+          break;
+      }	       
+    }
+
+    if (pos >= SWITCHER_N_SLOTS)
+      make_it_blink = TRUE;	     
   }	  
 	  
   children =
@@ -1940,7 +2040,7 @@ hd_switcher_menu_changed_info_cb (HDWM *hdwm,
       switcher->priv->last_urgent_info == info)	 /* application's one. Now we've been told to change it back*/
   {
     hd_switcher_menu_reset_main_icon (switcher);	  
-    switcher->priv->last_urgent_info = NULL;	  
+    switcher->priv->last_urgent_info = NULL;
   }	  
 
   if (hd_wm_entry_info_is_urgent (info) &&
@@ -1954,9 +2054,9 @@ hd_switcher_menu_changed_info_cb (HDWM *hdwm,
 
       if (make_it_blink)
       {	      
-        hd_switcher_menu_replace_blinking_icon 
+        hd_switcher_menu_replace_blinking_icon
   	  (switcher, hd_switcher_get_default_icon_from_entry_info (switcher,info));
-	
+
 	switcher->priv->last_urgent_info = info;
       }
     }
@@ -1995,10 +2095,20 @@ hd_switcher_menu_changed_stack_cb (HDWM *hdwm,
 static void 
 hd_switcher_menu_show_menu_cb (HDWM *hdwm, HDSwitcherMenu *switcher)
 {
-  gtk_toggle_button_set_active
-    (GTK_TOGGLE_BUTTON (switcher->priv->toggle_button), TRUE);
+  if (switcher->priv->is_open)
+  {	  
+    hildon_desktop_popup_menu_select_next_item (switcher->priv->menu_applications);
+    hd_wm_debug ("selecting next item");
+  }
+  else
+  {
+    gtk_window_close_other_temporaries (GTK_WINDOW (switcher->priv->popup_window));
 
-  g_signal_emit_by_name (switcher->priv->toggle_button, "toggled");
+    gtk_toggle_button_set_active
+      (GTK_TOGGLE_BUTTON (switcher->priv->toggle_button), TRUE);
+  
+    g_signal_emit_by_name (switcher->priv->toggle_button, "toggled");
+  }
 }
 
 static gboolean
@@ -2074,7 +2184,7 @@ hd_switcher_menu_long_press_cb (HDWM *hdwm, HDSwitcherMenu *switcher)
          (switcher->priv->menu_applications));
   }
   else
-    hd_wm_top_desktop ();
+    hd_wm_toggle_desktop ();
 }
 
 static void 
@@ -2186,6 +2296,9 @@ hd_switcher_menu_notification_deleted_cb (HildonDesktopNotificationManager   *nm
 
       if (menu_item)
       {
+        if (switcher->priv->blinking_notification == id)
+	  hd_switcher_menu_reset_main_icon (switcher);
+	
         hildon_desktop_popup_menu_remove_item (switcher->priv->menu_notifications,
 	                                       menu_item);
 
